@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -233,16 +233,17 @@ void PMLExplorerWidget::selectItems() {
             if (Cell* cellSelected = dynamic_cast<Cell*>(comp)) {        // Is it a cell ?
                 pmlComponent->selectCell(cellSelected, showAtomGlyph);
             }
-            else if (MultiComponent* mc = dynamic_cast<MultiComponent*>(comp)) {          // Is it a MC ?
+            else if (MultiComponent* mc = dynamic_cast<MultiComponent*>(comp)) {         // Is it a MC ?
                 pmlComponent->selectMC(mc, showAtomGlyph);
             }
-            else if (StructuralComponent* sc = dynamic_cast<StructuralComponent*>(comp)) {          // Is it a SC ?
+            else if (StructuralComponent* sc = dynamic_cast<StructuralComponent*>(comp)) {         // Is it a SC ?
                 pmlComponent->selectSC(sc, showAtomGlyph);
             }
 
             // create the corresponding QObject
             updateProperty(item);
         }
+
         // in any case (even for item == physicalModelItem), update the property
         updateProperty(item);
     }
@@ -265,6 +266,7 @@ void PMLExplorerWidget::updateTarget(PMLComponent* comp) {
     foreach (QObject* o, widgetPropertiesMap) {
         delete o;
     }
+
     foreach (QObject* o, atomPropertiesMap) {
         delete o;
     }
@@ -328,6 +330,7 @@ void PMLExplorerWidget::updateProperty(QTreeWidgetItem* item) {
     if (!itemProperties) {
         if (widgetAtomMap.contains(item)) {
             Atom* a = widgetAtomMap.value(item);
+
             if (a) {
                 // check if this atom already has an list of properties
                 itemProperties = atomPropertiesMap.value(a);
@@ -363,6 +366,7 @@ void PMLExplorerWidget::updateProperty(QTreeWidgetItem* item) {
             editedItem = nullptr;
         }
     }
+
     // update the ObjectController
     editedItem = item;
     objectController->setObject(itemProperties);
@@ -384,6 +388,7 @@ void PMLExplorerWidget::createNewProperty() {
             QString propName = addNewPropertyUI.nameLineEdit->text();
             QString propValue = addNewPropertyUI.valueLineEdit->text();
             camitk::Property* newProp = new camitk::Property(propName, propValue, "", "");
+
             if (editedItem == physicalModelItem) {
                 physicalModel->getProperties()->set(propName.toStdString(), propValue.toStdString());
                 pmlComponent->addProperty(newProp);
@@ -393,20 +398,27 @@ void PMLExplorerWidget::createNewProperty() {
             }
             else {
                 QObject* editedObject = nullptr;
+
                 if (Atom* a = widgetAtomMap.value(editedItem)) {
                     editedObject = atomPropertiesMap.value(a);
                 }
                 else if (widgetComponentMap.contains(editedItem)) {
                     editedObject = widgetPropertiesMap.value(editedItem);
                 }
+
                 // the cast should always be ok
                 auto* propList = dynamic_cast<CamiTKPropertyList*>(editedObject);
+
                 if (propList != nullptr) {
                     propList->addProperty(newProp);
                     // update object controller
-                    objectController->setObject(NULL);
+                    objectController->setObject(nullptr);
                     objectController->setObject(propList);
                 }
+                else {
+                    // to prevent compiler warning about potential memory link (should never come to that)
+                    delete newProp;
+                }
             }
 
         }
@@ -430,6 +442,7 @@ CamiTKPropertyList* PMLExplorerWidget::createAtomPropertyList(Atom* a) {
 
     // atom name
     QString atomName(a->getName().c_str());
+
     if (atomName != "") {
         itemProperties->addProperty(new camitk::Property("name", atomName, "(optional) name of this atom", ""));
     }
@@ -442,19 +455,23 @@ CamiTKPropertyList* PMLExplorerWidget::createAtomPropertyList(Atom* a) {
     // add all data array value
     vtkSmartPointer<vtkPointData> allPointData = pmlComponent->getPointSet()->GetPointData();
     std::string activeDataName;
+
     if (allPointData->GetScalars()) {
         activeDataName = allPointData->GetScalars()->GetName();
     }
+
     for (int i = 0; i < allPointData->GetNumberOfArrays(); i++) {
         vtkSmartPointer<vtkDataArray> array = allPointData->GetArray(i);
         std::string arrayName = array->GetName();
         std::string dataName = "Monitoring Data";
         bool isActive = (activeDataName.compare(arrayName) == 0);
+
         // check if this array is the active one
         if (isActive) {
             arrayName += " (*)";
             dataName += " (active/displayed data point)";
         }
+
         double* val = array->GetTuple(pmlComponent->getPointId(a));
         camitk::Property* dataProp = new camitk::Property(arrayName.c_str(), *val, dataName.c_str(), "");
         dataProp->setGroupName("Monitors");
@@ -469,7 +486,7 @@ CamiTKPropertyList* PMLExplorerWidget::createCellPropertyList(Cell* c) {
     auto* itemProperties = new CamiTKPropertyList;
 
     // index
-    camitk::Property* indexProp = new camitk::Property("index", QVariant((int) c->getIndex()), "Unique index of the cell (can be totally arbirtrary, cell indexes do not have to be contiguous)", "unsigned int");
+    camitk::Property* indexProp = new camitk::Property("index", QVariant((int) c->getIndex()), "Unique index of the cell (can be totally arbitrary, cell indexes do not have to be contiguous)", "unsigned int");
     indexProp->setReadOnly(true);
     itemProperties->addProperty(indexProp);
 
@@ -484,6 +501,7 @@ CamiTKPropertyList* PMLExplorerWidget::createCellPropertyList(Cell* c) {
 
     // cell name
     QString cellName(((Structure*) c)->getName().c_str());
+
     if (cellName != "") {
         itemProperties->addProperty(new camitk::Property("name", cellName, "(optional) name of this cell", ""));
     }
@@ -511,22 +529,27 @@ CamiTKPropertyList* PMLExplorerWidget::createSCPropertyList(StructuralComponent*
 
     // sc name
     QString scName(sc->getName().c_str());
+
     if (scName != "") {
         itemProperties->addProperty(new camitk::Property("name", scName, "(optional) name of this structural component", ""));
     }
 
     // composed by
     QString composedByValue;
+
     switch (sc->composedBy()) {
         case StructuralComponent::ATOMS:
             composedByValue = "Atoms";
             break;
+
         case StructuralComponent::CELLS:
             composedByValue = "Cells";
             break;
+
         default:
             composedByValue = "Nothing !";
     }
+
     camitk::Property* composedByProp = new camitk::Property("Composed By", composedByValue, "Nature of the structures (cells or atoms)", "");
     composedByProp->setReadOnly(true);
     itemProperties->addProperty(composedByProp);
@@ -558,6 +581,7 @@ CamiTKPropertyList* PMLExplorerWidget::createMCPropertyList(MultiComponent* mc)
 
     // mc name
     QString mcName(mc->getName().c_str());
+
     if (mcName != "") {
         itemProperties->addProperty(new camitk::Property("name", mcName, "(optional) name of this multi-component", ""));
     }
@@ -593,6 +617,7 @@ void PMLExplorerWidget::propertyChanged() {
             a->setPosition(pos.x(), pos.y(), pos.z());
 
             QString name = propList->property("name").value<QString>();
+
             if (!name.isEmpty()) {
                 a->setName(name.toStdString());
             }
@@ -603,6 +628,7 @@ void PMLExplorerWidget::propertyChanged() {
             // all other properties
             for (unsigned int i = 0; i < propList->getNumberOfProperties(); i++) {
                 QString propName = propList->getProperty(i)->getName();
+
                 if (propName != "index" &&  propName != "Position" && propName != "name" && !propList->getProperty(i)->getReadOnly()) {
                     // any other property
                     QString value = propList->property(propName.toStdString().c_str()).value<QString>();
@@ -621,11 +647,15 @@ void PMLExplorerWidget::propertyChanged() {
             Component* comp = widgetComponentMap.value(editedItem);
             QString name = propList->property("name").value<QString>();
 
-            if (Cell* c = dynamic_cast<Cell*>(comp)) {
+            Cell* c = dynamic_cast<Cell*>(comp);
+
+            if (c != nullptr) {
                 ((Structure*) c)->setName(name.toStdString());
+
                 // all other properties
                 for (unsigned int i = 0; i < propList->getNumberOfProperties(); i++) {
                     QString propName = propList->getProperty(i)->getName();
+
                     if (propName != "name" && !propList->getProperty(i)->getReadOnly()) {
                         // any other property
                         QString value = propList->property(propName.toStdString().c_str()).value<QString>();
@@ -634,14 +664,18 @@ void PMLExplorerWidget::propertyChanged() {
                 }
             }
             else {
-                comp->setName(name.toStdString());
-                // all other properties
-                for (unsigned int i = 0; i < propList->getNumberOfProperties(); i++) {
-                    QString propName = propList->getProperty(i)->getName();
-                    if (propName != "name" && !propList->getProperty(i)->getReadOnly()) {
-                        // any other property
-                        QString value = propList->property(propName.toStdString().c_str()).value<QString>();
-                        comp->getProperties()->set(propName.toStdString(), value.toStdString());
+                if (comp != nullptr) {
+                    comp->setName(name.toStdString());
+
+                    // all other properties
+                    for (unsigned int i = 0; i < propList->getNumberOfProperties(); i++) {
+                        QString propName = propList->getProperty(i)->getName();
+
+                        if (propName != "name" && !propList->getProperty(i)->getReadOnly()) {
+                            // any other property
+                            QString value = propList->property(propName.toStdString().c_str()).value<QString>();
+                            comp->getProperties()->set(propName.toStdString(), value.toStdString());
+                        }
                     }
                 }
             }
