@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -149,7 +149,7 @@ Cell* Facet::getCell(PhysicalModel* pm) const {
     // and insert them in the cell
     for (unsigned int i = 0; i < size; i++) {
         Atom* a = pm->getAtom(id[i]);
-        if (a == NULL) {
+        if (a == nullptr) {
             cout << "Argh! Cannot find atom #" << id[i] << endl;
         }
         else {
@@ -179,7 +179,7 @@ map<unsigned int, Cell*>::iterator getIterator(unsigned int index) {
 
     // not present, insert a new cell, associated with index
     if (it == neighMap.end()) {
-        // instanciate
+        // instantiate
         Cell* nc = new Cell(NULL, StructureProperties::POLY_VERTEX);
         // name
         stringstream n(std::stringstream::out);
@@ -420,7 +420,7 @@ StructuralComponent* generateNeighborhood(StructuralComponent* sc) {
     // now that we have in neighMap all the neighborhoods, just add them in a new SC
     StructuralComponent* neigh = new StructuralComponent(NULL, "Neighborhoods");
 
-    for (it = neighMap.begin(); it != neighMap.end(); it++) {
+    for (it = neighMap.begin(); it != neighMap.end(); ++it) {
         neigh->addStructure((*it).second);
     }
 
@@ -439,7 +439,7 @@ void equivalent(int size, unsigned int id[]) {
     // look into allFacets for equivalence
     it = allFacets.begin();
     while (it != allFacets.end() && !(*it)->testEquivalence(size, id)) {
-        it++;
+        ++it;
     }
 
     // not found => insert
@@ -622,7 +622,7 @@ MultiComponent* generateExternalSurface(StructuralComponent* sc) {
     // just add in a new SC all the one that are used only once
     StructuralComponent* facet = new StructuralComponent(NULL, "extern");
 
-    for (vector <Facet*>::iterator it = allFacets.begin(); it != allFacets.end(); it++) {
+    for (vector <Facet*>::iterator it = allFacets.begin(); it != allFacets.end(); ++it) {
         //(*it)->debug();
         if ((*it)->getUsed() == 1) { // used only once
             facet->addStructure((*it)->getCell(sc->getPhysicalModel()));
@@ -630,7 +630,7 @@ MultiComponent* generateExternalSurface(StructuralComponent* sc) {
     }
 
     // delete all facets
-    for (vector <Facet*>::iterator it = allFacets.begin(); it != allFacets.end(); it++) {
+    for (vector <Facet*>::iterator it = allFacets.begin(); it != allFacets.end(); ++it) {
         delete (*it);
     }
 
