@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -307,7 +307,7 @@ Connection* insert(Connection* f) {
     // look into allConnections for equivalence
     it = allConnections.begin();
     while (it != allConnections.end() && !equivalent((*it), f)) {
-        it++;
+        ++it;
     }
 
     // not found => insert
