@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -249,7 +249,7 @@ Cluster* insert(Atom* a) {
     // look into allClusters for equivalence
     it = allClusters.begin();
     while (it != allClusters.end() && !(*it)->addAtom(a)) {
-        it++;
+        ++it;
     }
 
     // not found => insert
@@ -268,7 +268,7 @@ Cluster* insert(Atom* a) {
 
 // -------------------- update ------------------------
 void update(MultiComponent* mc) {
-    if (mc == NULL) {
+    if (mc == nullptr) {
         cout << "0 cells)" << endl;
         return;
     }
@@ -353,7 +353,7 @@ void saveClusters(string clusterFile) {
     cout << "-> please wait while saving cluster informations in " << clusterFile << " (" << pm->getNumberOfAtoms() - allClusters.size() << " clusters) and non-clusters in " << nonClusterFile << endl;
     ofstream clusterOutputFile(clusterFile.c_str());
     ofstream nonClusterOutputFile(nonClusterFile.c_str());
-    for (vector <Cluster*>::iterator it = allClusters.begin(); it != allClusters.end(); it++) {
+    for (vector <Cluster*>::iterator it = allClusters.begin(); it != allClusters.end(); ++it) {
         if ((*it)->nrOfAtoms() > 1) {
             clusterOutputFile << (*it)->getCenter()->getIndex() << " : ";
             for (unsigned int i = 0; i < (*it)->nrOfAtoms(); i++) {
@@ -386,7 +386,7 @@ void optimize(string clusterFile) {
     // create newAtoms with all atoms that are alone in their clusters and
     // all cluster centers
     StructuralComponent* newAtoms = new StructuralComponent(pm);
-    for (vector <Cluster*>::iterator it = allClusters.begin(); it != allClusters.end(); it++) {
+    for (vector <Cluster*>::iterator it = allClusters.begin(); it != allClusters.end(); ++it) {
         if ((*it)->nrOfAtoms() == 1) {
             newAtoms->addStructure((*it)->getAtom(0));
             replacement.insert(pair<Atom*, Atom*>((*it)->getAtom(0), (*it)->getAtom(0)));
@@ -532,7 +532,7 @@ Cell* Facet::getCell(PhysicalModel* pm) const {
     // and insert them in the cell
     for (unsigned int i = 0; i < size; i++) {
         Atom* a = pm->getAtom(id[i]);
-        if (a == NULL) {
+        if (a == nullptr) {
             cout << "Argh! Cannot find atom #" << id[i] << endl;
         }
         else {
@@ -562,7 +562,7 @@ map<unsigned int, Cell*>::iterator getIterator(unsigned int index) {
 
     // not present, insert a new cell, associated with index
     if (it == neighMap.end()) {
-        // instanciate
+        // instantiate
         Cell* nc = new Cell(NULL, StructureProperties::POLY_VERTEX);
         // name
         stringstream n(std::stringstream::out);
@@ -803,7 +803,7 @@ StructuralComponent* generateNeighborhood(StructuralComponent* sc) {
     // now that we have in neighMap all the neighborhoods, just add them in a new SC
     StructuralComponent* neigh = new StructuralComponent(NULL, "Neighborhoods");
 
-    for (it = neighMap.begin(); it != neighMap.end(); it++) {
+    for (it = neighMap.begin(); it != neighMap.end(); ++it) {
         neigh->addStructure((*it).second);
     }
 
@@ -822,7 +822,7 @@ void equivalent(int size, unsigned int id[]) {
     // look into allFacets for equivalence
     it = allFacets.begin();
     while (it != allFacets.end() && !(*it)->testEquivalence(size, id)) {
-        it++;
+        ++it;
     }
 
     // not found => insert
@@ -1005,7 +1005,7 @@ MultiComponent* generateExternalSurface(StructuralComponent* sc) {
     // just add in a new SC all the one that are used only once
     StructuralComponent* facet = new StructuralComponent(NULL, "extern");
 
-    for (vector <Facet*>::iterator it = allFacets.begin(); it != allFacets.end(); it++) {
+    for (vector <Facet*>::iterator it = allFacets.begin(); it != allFacets.end(); ++it) {
         //(*it)->debug();
         if ((*it)->getUsed() == 1) { // used only once
             facet->addStructure((*it)->getCell(sc->getPhysicalModel()));
@@ -1013,7 +1013,7 @@ MultiComponent* generateExternalSurface(StructuralComponent* sc) {
     }
 
     // delete all facets
-    for (vector <Facet*>::iterator it = allFacets.begin(); it != allFacets.end(); it++) {
+    for (vector <Facet*>::iterator it = allFacets.begin(); it != allFacets.end(); ++it) {
         delete (*it);
     }
 
