@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -274,9 +274,7 @@ SofaSimulator::SofaSimulator(MonitoringManager* monitoringManager): InteractiveS
 }
 
 // -------------------- constructor --------------------
-SofaSimulator::SofaSimulator(MonitoringManager* monitoringManager, const char* file): InteractiveSimulator(monitoringManager, file) {
-    scnFile = string(file);
-
+SofaSimulator::SofaSimulator(MonitoringManager* monitoringManager, const char* file): InteractiveSimulator(monitoringManager, file), scnFile = string(file) {
     build();
 }
 
@@ -363,7 +361,7 @@ void SofaSimulator::build() {
     groot = dynamic_cast<sofa::simulation::Node*>(sofa::simulation::getSimulation()->load(scnFile.c_str()));
 #endif
 
-    if (getGNode() == NULL) {
+    if (getGNode() == nullptr) {
 #if defined(SOFA_1_0_RC1) || defined(SOFA_SVN) || defined(SOFA_STABLE)
         groot = sofa::simulation::tree::getSimulation()->createNewGraph("");
 #endif
@@ -560,7 +558,7 @@ void SofaSimulator::createPml(const char* inputFile, const char* pmlFile) {   //
     pm = new PhysicalModel();
     pm->setName(pmlName);
 
-    // instanciate the structural component
+    // instantiate the structural component
     StructuralComponent* atoms = new StructuralComponent(NULL, "DOF");
 
     // create the atom structures and create local and global maps between atoms and dof
@@ -614,7 +612,7 @@ void SofaSimulator::createPml(const char* inputFile, const char* pmlFile) {   //
         mechObjectMC->addSubComponent(sc);
     }
 
-    for (std::AtomDOFMap::iterator it = atomsToDOF.begin(); it != atomsToDOF.end(); it++) {
+    for (std::AtomDOFMap::iterator it = atomsToDOF.begin(); it != atomsToDOF.end(); ++it) {
         moSC[(it->second).first]->addStructureIfNotIn(pm->getAtom(it->first));
     }
 
