@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -129,7 +129,7 @@ Cell::~Cell() {
     StructuralComponent::atomList = nullptr;
 
     // delete all children
-    deleteAllStructures();
+    Cell::deleteAllStructures();
 
     // tell all parents that I am going away to the paradise of pointers
     removeFromParents();
@@ -376,6 +376,7 @@ StructuralComponent* Cell::getTriangleFaces() {
             facets->addStructure(this);
             return facets;
         default :
+            delete facets;
             return nullptr;
     }
 }
@@ -449,6 +450,7 @@ StructuralComponent* Cell::getQuadFaces() {
             facets->addStructure(this);
             return facets;
         default :
+            delete facets;
             return nullptr;
     }
 
@@ -517,6 +519,7 @@ double Cell::surface() {
         double* N = normal();
 
         double surface = N[0] * A[0] + N[1] * A[1] + N[2] * A[2];
+        delete []N;
 
         return surface > 0 ? surface : -surface;
     }
@@ -577,6 +580,7 @@ double Cell::volume() {
             ((Atom*) face->getStructure(0))->getPosition(pos);
             double* N = face->normal();
             vol += face->surface() * (pos[0] * N[0] + pos[1] * N[1] + pos[2] * N[2]);
+            delete [] N;
         }
         vol /= 3.0;
         if (vol < 0) {
