@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -136,7 +136,9 @@ Cell* Facet::getCell(PhysicalModel* pm) const {
             std::cout << "Argh! Cannot find atom #" << id[i] << std::endl;
         }
         else {
-            c->addStructureIfNotIn(a);
+            if (c != nullptr) {
+                c->addStructureIfNotIn(a);
+            }
         }
     }
 
@@ -168,7 +170,7 @@ std::map<unsigned int, Cell*>::iterator PMLTransform::getIterator(unsigned int i
 
     // not present, insert a new cell, associated with index
     if (it == neighMap.end()) {
-        // instanciate
+        // instantiate
         Cell* nc = new Cell(nullptr, StructureProperties::POLY_VERTEX);
         // name
         std::stringstream n(std::stringstream::out);
@@ -346,7 +348,7 @@ StructuralComponent* PMLTransform::generateNeighborhood(StructuralComponent* sc)
     // now that we have in neighMap all the neighborhoods, just add them in a new SC
     StructuralComponent* neigh = new StructuralComponent(nullptr, "Neighborhoods");
 
-    for (it = neighMap.begin(); it != neighMap.end(); it++) {
+    for (it = neighMap.begin(); it != neighMap.end(); ++it) {
         neigh->addStructure((*it).second);
     }
 
@@ -363,7 +365,7 @@ void PMLTransform::equivalent(int size, unsigned int id[]) {
     it = allFacets.begin();
 
     while (it != allFacets.end() && !(*it)->testEquivalence(size, id)) {
-        it++;
+        ++it;
     }
 
     // not found => insert
