@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -266,7 +266,7 @@ void PhysicalModel::xmlPrint(std::ostream& o, bool opt) {
     o << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << std::endl;
 
     o << "<!-- physical model (PML) is a generic representation for 3D physical model." << std::endl
-      << "     PML supports not continous indexes and multiple non-exclusive labelling." << std::endl
+      << "     PML supports not continuous indexes and multiple non-exclusive labelling." << std::endl
       << "  --> " << std::endl;
 
     o << "<physicalModel";
@@ -371,7 +371,7 @@ void PhysicalModel::xmlRead(const char* filename) {
         // get the basename
         std::string basename = filename;
         // remove the path
-        unsigned lastSeparator = basename.find_last_of("/\\");    // works for unix and windows
+        auto lastSeparator = basename.find_last_of("/\\");    // works for unix and windows
         if (lastSeparator != std::string::npos) {
             basename = basename.substr(lastSeparator + 1);
         }
@@ -617,7 +617,7 @@ bool PhysicalModel::addGlobalIndexAtomPair(std::GlobalIndexStructurePair p) {
     mapIt = atomMap.begin();
 
     while (mapIt != atomMap.end() && mapIt->second != p.second) {
-        mapIt++;
+        ++mapIt;
     }
 
     // if found then remove the pair
@@ -645,7 +645,7 @@ bool PhysicalModel::addGlobalIndexCellPair(std::GlobalIndexStructurePair p) {
     mapIt = cellMap.begin();
 
     while (mapIt != cellMap.end() && mapIt->second != p.second) {
-        mapIt++;
+        ++mapIt;
     }
 
     // if found then remove the pair
@@ -882,7 +882,7 @@ void PhysicalModel::exportAnsysMesh(std::string filename) {
 
 
             default:
-                std::cerr << "PhysicalModel::exportAnsysMesh : unknown type for cell " << cell->getIndex() + 1 << ", neither HEXAHEDRON, WEDGE, THETRAHEDRON nor QUAD. Cant' export in Patran format." << std::endl;
+                std::cerr << "PhysicalModel::exportAnsysMesh : unknown type for cell " << cell->getIndex() + 1 << ", neither HEXAHEDRON, WEDGE, TETRAHEDRON nor QUAD. Cant' export in Patran format." << std::endl;
                 continue;
         }
 
