@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -25,14 +25,11 @@
 #include "ArbitrarySliceExtension.h"
 
 // include generated actions headers
-#include "SetAnglesAction.h"
-
-// --------------- declare the extension -------------------
-Q_EXPORT_PLUGIN2(arbitraryslice, ArbitrarySliceExtension)
+#include "AnglesAndTranslationAction.h"
 
 // --------------- getActions -------------------
 void ArbitrarySliceExtension::init() {
     // Creating and registering the instance of AdjustArbiratrySliceAnglesAction
-    registerNewAction(SetAnglesAction);
+    registerNewAction(AnglesAndTranslationAction);
 }
 
