@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -42,52 +42,74 @@
 using namespace camitk;
 
 // ---------------------- Constructor ----------------------
-VolumeRenderingWidget::VolumeRenderingWidget(VolumeRenderingAction* dad, QWidget* parent) : QWidget(parent) {
-    this->dad = dad;
-    myImage = NULL;
+VolumeRenderingWidget::VolumeRenderingWidget(VolumeRenderingAction* myAction, QWidget* parent) : QWidget(parent) {
+    this->myAction = myAction;
     ui.setupUi(this);
 }
 
 // ---------------------- Destructor ----------------------
 VolumeRenderingWidget::~VolumeRenderingWidget() {
+    // TODO manage this when the colormap changes
     // Remove all existing colors
     QList<ColorPointWidget*>::const_iterator colorIt;
-    for (colorIt = colorPoints.begin(); colorIt != colorPoints.end(); colorIt++) {
+
+    for (colorIt = colorPoints.constBegin(); colorIt != colorPoints.constEnd(); ++colorIt) {
         ColorPointWidget* point = (*colorIt);
         ui.colorsLayout->removeWidget(point);
         delete point;
     }
+
     colorPoints.clear();
 
     // Remove all existing transparencies
     QList<TransparencyPointWidget*>::const_iterator transIt;
-    for (transIt = transparencies.begin(); transIt != transparencies.end(); transIt++) {
+
+    for (transIt = transparencies.constBegin(); transIt != transparencies.constEnd(); ++transIt) {
         TransparencyPointWidget* point = (*transIt);
         ui.transparenciesLayout->removeWidget(point);
         delete point;
     }
+
     gradientOpacities.clear();
     // Remove all existing grandient opacities
     QList<GradientOpacityWidget*>::const_iterator gradientIt;
-    for (gradientIt = gradientOpacities.begin(); gradientIt != gradientOpacities.end(); gradientIt++) {
+
+    for (gradientIt = gradientOpacities.constBegin(); gradientIt != gradientOpacities.constEnd(); ++gradientIt) {
         GradientOpacityWidget* point = (*gradientIt);
         ui.gradientOpacitiesLayout->removeWidget(point);
         delete point;
     }
+
     gradientOpacities.clear();
 }
 
 // ---------------------- updateImage ----------------------
-void VolumeRenderingWidget::updateImage(ImageComponent* comp) {
-    if (myImage != comp) {
-        myImage = comp;
-        ui.imageNameLabel->setText(myImage->getName());
-        QString filename(":resources/default.clm");
-        load(filename);
+void VolumeRenderingWidget::updateUI(QString name, bool existVR) {
+    ui.imageNameLabel->setText(name);
+    ui.showCheckBox->setChecked(existVR && myAction->property("Volume Rendering Visibility").toBool());
 
+    // TODO create a VRColorMap class to manage load/save and add a QMap<QString*,VRColorMap*> to save
+    // the map when the user switch between different images
+    // note : see vrColorMap.xsd and default.clm, move the XML read/write from load/save to the new
+    // VRColorMap class.
+    // TODO create a list of nice default colormaps for TDM and MRI
+    // TODO add a popup that will load the default colormap (instead of the refresh button)
+    if (!existVR) {
+        resetColorMap();
     }
 }
 
+// ---------------------- visibilityToggled ----------------------
+void VolumeRenderingWidget::visibilityToggled(bool visibility) {
+    myAction->setProperty("Volume Rendering Visibility", visibility);
+}
+
+// ---------------------- refreshVolumeRendering ----------------------
+void VolumeRenderingWidget::refreshVolumeRendering() {
+    // the user clicked on the refresh button, just signal it
+    emit refresh();
+}
+
 // ---------------------- addColor ----------------------
 void VolumeRenderingWidget::addColor() {
     ColorPointWidget* point = new ColorPointWidget(this);
@@ -112,31 +134,32 @@ void VolumeRenderingWidget::removeColorPoint(ColorPointWidget* point) {
 void VolumeRenderingWidget::setDefaultColors(QMap<int, QColor> defaultColors) {
     // Remove all existing colors
     QList<ColorPointWidget*>::const_iterator colorIt;
-    for (colorIt = colorPoints.begin(); colorIt != colorPoints.end(); colorIt++) {
+
+    for (colorIt = colorPoints.constBegin(); colorIt != colorPoints.constEnd(); ++colorIt) {
         ColorPointWidget* point = (*colorIt);
         ui.colorsLayout->removeWidget(point);
         delete point;
     }
+
     colorPoints.clear();
 
     // Add default colors given in parameters
     QMap<int, QColor>::const_iterator it;
-    for (it = defaultColors.begin(); it != defaultColors.end(); it++) {
+
+    for (it = defaultColors.constBegin(); it != defaultColors.constEnd(); ++it) {
         int gray = it.key();
         QColor color = it.value();
         addColor(gray, color);
     }
+
 }
 
 // ---------------------- getColorPoints ----------------------
 QMap<int, QColor> VolumeRenderingWidget::getColorPoints() {
     QMap<int, QColor> colors;
-    QList<ColorPointWidget*>::const_iterator it;
-    for (it = colorPoints.begin(); it != colorPoints.end(); it++) {
-        int gray = (*it)->getGrayLevel();
-        QColor color = (*it)->getColor();
 
-        colors.insert(gray, color);
+    for (QList<ColorPointWidget*>::const_iterator it = colorPoints.constBegin(); it != colorPoints.constEnd(); ++it) {
+        colors.insert((*it)->getGrayLevel(), (*it)->getColor());
     }
 
     return colors;
@@ -166,16 +189,19 @@ void VolumeRenderingWidget::removeTransparencyPoint(TransparencyPointWidget* poi
 void VolumeRenderingWidget::setDefaultTransparencies(QMap<int, double> defaultTransparencies) {
     // Remove all existing transparencies
     QList<TransparencyPointWidget*>::const_iterator transIt;
-    for (transIt = transparencies.begin(); transIt != transparencies.end(); transIt++) {
+
+    for (transIt = transparencies.constBegin(); transIt != transparencies.constEnd(); ++transIt) {
         TransparencyPointWidget* point = (*transIt);
         ui.transparenciesLayout->removeWidget(point);
         delete point;
     }
+
     transparencies.clear();
 
     // Add default colors given in parameters
     QMap<int, double>::const_iterator it;
-    for (it = defaultTransparencies.begin(); it != defaultTransparencies.end(); it++) {
+
+    for (it = defaultTransparencies.constBegin(); it != defaultTransparencies.constEnd(); ++it) {
         int gray = it.key();
         double transparency = it.value();
         addTransparency(gray, transparency);
@@ -185,12 +211,9 @@ void VolumeRenderingWidget::setDefaultTransparencies(QMap<int, double> defaultTr
 // ---------------------- getTransparencyPoints ----------------------
 QMap<int, double> VolumeRenderingWidget::getTransparencyPoints() {
     QMap<int, double> transPoints;
-    QList<TransparencyPointWidget*>::const_iterator it;
-    for (it = transparencies.begin(); it != transparencies.end(); it++) {
-        int gray = (*it)->getGrayLevel();
-        double transparency = (*it)->getTransparency();
 
-        transPoints.insert(gray, transparency);
+    for (QList<TransparencyPointWidget*>::const_iterator it = transparencies.constBegin(); it != transparencies.constEnd(); ++it) {
+        transPoints.insert((*it)->getGrayLevel(), (*it)->getTransparency());
     }
 
     return transPoints;
@@ -220,16 +243,19 @@ void VolumeRenderingWidget::removeOpacityPoint(GradientOpacityWidget* point) {
 void VolumeRenderingWidget::setDefaultOpacities(QMap<int, double> defaultOpacities) {
     // Remove all existing colors
     QList<GradientOpacityWidget*>::const_iterator gradientIt;
-    for (gradientIt = gradientOpacities.begin(); gradientIt != gradientOpacities.end(); gradientIt++) {
+
+    for (gradientIt = gradientOpacities.constBegin(); gradientIt != gradientOpacities.constEnd(); ++gradientIt) {
         GradientOpacityWidget* point = (*gradientIt);
         ui.gradientOpacitiesLayout->removeWidget(point);
         delete point;
     }
+
     gradientOpacities.clear();
 
     // Add default colors given in parameters
     QMap<int, double>::const_iterator it;
-    for (it = defaultOpacities.begin(); it != defaultOpacities.end(); it++) {
+
+    for (it = defaultOpacities.constBegin(); it != defaultOpacities.constEnd(); ++it) {
         int gray = it.key();
         double opacity = it.value();
         addGradientOpacity(gray, opacity);
@@ -239,26 +265,22 @@ void VolumeRenderingWidget::setDefaultOpacities(QMap<int, double> defaultOpaciti
 // ---------------------- getOpacityPoints ----------------------
 QMap<int, double> VolumeRenderingWidget::getOpacityPoints() {
     QMap<int, double> opacities;
-    QList<GradientOpacityWidget*>::const_iterator it;
-    for (it = gradientOpacities.begin(); it != gradientOpacities.end(); it++) {
-        int gray = (*it)->getGrayLevel();
-        double opacity = (*it)->getOpacity();
 
-        opacities.insert(gray, opacity);
+    for (QList<GradientOpacityWidget*>::const_iterator it = gradientOpacities.constBegin(); it != gradientOpacities.constEnd(); ++it) {
+        opacities.insert((*it)->getGrayLevel(), (*it)->getOpacity());
     }
 
     return opacities;
 }
 
-
-// ---------------------- setAmbiant ----------------------
-void VolumeRenderingWidget::setAmbiant(double ambiant) {
-    ui.ambiantSpinBox->setValue(ambiant);
+// ---------------------- setambient ----------------------
+void VolumeRenderingWidget::setAmbient(double ambient) {
+    ui.ambientSpinBox->setValue(ambient);
 }
 
-// ---------------------- getAmbiant ----------------------
-double VolumeRenderingWidget::getAmbiant() {
-    return ui.ambiantSpinBox->value();
+// ---------------------- getambient ----------------------
+double VolumeRenderingWidget::getAmbient() {
+    return ui.ambientSpinBox->value();
 }
 
 // ---------------------- setDiffuse ----------------------
@@ -283,10 +305,10 @@ double VolumeRenderingWidget::getSpecular() {
 
 // ---------------------- load ----------------------
 void VolumeRenderingWidget::load(QString filename) {
-    QString msg;
     QDomDocument doc;
 
     QFile file(filename);
+
     if (!file.open(QIODevice::ReadOnly)) {
         CAMITK_ERROR(tr("File not found: \"%1\"").arg(filename))
         return;
@@ -299,6 +321,7 @@ void VolumeRenderingWidget::load(QString filename) {
     }
 
     QString rootName = doc.documentElement().nodeName();
+
     if (rootName != QString("vrColorMap")) {
         file.close();
         CAMITK_ERROR(tr("File \"%1\" is not a valid XML document (expecting <vrColorMap> as root element).").arg(filename))
@@ -312,15 +335,15 @@ void VolumeRenderingWidget::load(QString filename) {
     QMap<int, double> defaultTransparencies;
     QMap<int, QColor> defaultColors;
     QMap<int, double> defaultGradientOpacities;
-    double defaultAmbiant;
+    double defaultambient;
     double defaultDiffuse;
     double defaultSpecular;
 
-
     QDomElement docElem = doc.documentElement();
 
     QDomNodeList xmlTransparencies = docElem.elementsByTagName("transparencies");
     QDomNodeList transparencyList = xmlTransparencies.item(0).toElement().elementsByTagName("transparency");
+
     for (int i = 0; i < transparencyList.size(); i++) {
         QDomElement aTransaprency = transparencyList.item(i).toElement();
         int gray = aTransaprency.attribute("gray").toInt();
@@ -328,9 +351,9 @@ void VolumeRenderingWidget::load(QString filename) {
         defaultTransparencies.insert(gray, value);
     }
 
-
     QDomNodeList xmlColors = docElem.elementsByTagName("colors");
     QDomNodeList colorList = xmlColors.item(0).toElement().elementsByTagName("color");
+
     for (int i = 0; i < colorList.size(); i++) {
         QDomElement aColor = colorList.item(i).toElement();
         int gray = aColor.attribute("gray").toInt();
@@ -342,9 +365,9 @@ void VolumeRenderingWidget::load(QString filename) {
         defaultColors.insert(gray, QColor(red, green, blue, alpha));
     }
 
-
     QDomNodeList xmlGradientOpacities = docElem.elementsByTagName("gradientOpacities");
     QDomNodeList gradientOpacitiesList = xmlGradientOpacities.item(0).toElement().elementsByTagName("gradientOpacity");
+
     for (int i = 0; i < gradientOpacitiesList.size(); i++) {
         QDomElement aTransaprency = gradientOpacitiesList.item(i).toElement();
         int gray = aTransaprency.attribute("gray").toInt();
@@ -353,8 +376,8 @@ void VolumeRenderingWidget::load(QString filename) {
     }
 
     QDomNodeList xmlShading = docElem.elementsByTagName("shading");
-    QDomElement xmlAmbiant = xmlShading.item(0).toElement().elementsByTagName("ambiant").item(0).toElement();
-    defaultAmbiant = xmlAmbiant.attribute("value").toDouble();
+    QDomElement xmlambient = xmlShading.item(0).toElement().elementsByTagName("ambient").item(0).toElement();
+    defaultambient = xmlambient.attribute("value").toDouble();
 
     QDomElement xmlDiffuse = xmlShading.item(0).toElement().elementsByTagName("diffuse").item(0).toElement();
     defaultDiffuse = xmlDiffuse.attribute("value").toDouble();
@@ -365,23 +388,26 @@ void VolumeRenderingWidget::load(QString filename) {
     setDefaultTransparencies(defaultTransparencies);
     setDefaultColors(defaultColors);
     setDefaultOpacities(defaultGradientOpacities);
-    ui.ambiantSpinBox->setValue(defaultAmbiant);
+    ui.ambientSpinBox->setValue(defaultambient);
     ui.diffuseSpinBox->setValue(defaultDiffuse);
     ui.specularSpinBox->setValue(defaultSpecular);
 
+    // update the action property
+    myAction->setProperty("Color Map File", filename);
 }
 
-void VolumeRenderingWidget::load() {
-    QString filename = QFileDialog::getOpenFileName(this, tr("Open Volume Rendering Colormap"));
+// ---------------------- loadColorMap ----------------------
+void VolumeRenderingWidget::loadColorMap() {
+    QString filename = QFileDialog::getOpenFileName(this, tr("Open a Volume Rendering Colormap XML Document"));
     load(filename);
 }
 
-// ---------------------- save ----------------------
-void VolumeRenderingWidget::save() {
-    QString filename = QFileDialog::getSaveFileName(this, tr("Save Volume Rendering Colormap"));
+// ---------------------- saveColorMap ----------------------
+void VolumeRenderingWidget::saveColorMap() {
+    QString filename = QFileDialog::getSaveFileName(this, tr("Save to a Volume Rendering Colormap XML Document"));
 
-    QString msg;
     QDomDocument doc;
+    doc.createProcessingInstruction("xml", "version=\"1.0\" encoding=\"utf-8\"");
     QDomElement racine = doc.createElement("vrColorMap");
     doc.appendChild(racine);
 
@@ -389,7 +415,8 @@ void VolumeRenderingWidget::save() {
     racine.appendChild(xmlTransparencies);
 
     QList<TransparencyPointWidget*>::const_iterator transIt;
-    for (transIt = transparencies.begin(); transIt != transparencies.end(); transIt++) {
+
+    for (transIt = transparencies.constBegin(); transIt != transparencies.constEnd(); ++transIt) {
         int gray = (*transIt)->getGrayLevel();
         double transparency = (*transIt)->getTransparency();
         QString tmp;// = QString:number(transparency, 'g', 3);
@@ -405,7 +432,8 @@ void VolumeRenderingWidget::save() {
     racine.appendChild(xmlColors);
 
     QList<ColorPointWidget*>::const_iterator colorIt;
-    for (colorIt = colorPoints.begin(); colorIt != colorPoints.end(); colorIt++) {
+
+    for (colorIt = colorPoints.constBegin(); colorIt != colorPoints.constEnd(); ++colorIt) {
         int gray = (*colorIt)->getGrayLevel();
         QColor color = (*colorIt)->getColor();
         QString tmp;
@@ -420,12 +448,12 @@ void VolumeRenderingWidget::save() {
         xmlColors.appendChild(trans);
     }
 
-
     QDomElement xmlGradientOpacities = doc.createElement("gradientOpacities");
     racine.appendChild(xmlGradientOpacities);
 
     QList<GradientOpacityWidget*>::const_iterator gradIt;
-    for (gradIt = gradientOpacities.begin(); gradIt != gradientOpacities.end(); gradIt++) {
+
+    for (gradIt = gradientOpacities.constBegin(); gradIt != gradientOpacities.constEnd(); ++gradIt) {
         int gray = (*gradIt)->getGrayLevel();
         double transparency = (*gradIt)->getOpacity();
         QString tmp;// = QString:number(transparency, 'g', 3);
@@ -441,9 +469,9 @@ void VolumeRenderingWidget::save() {
     racine.appendChild(xmlShading);
 
     QString shadeTmp;
-    QDomElement xmlAmbiant = doc.createElement("ambiant");
-    xmlAmbiant.setAttribute("value", shadeTmp.number(ui.ambiantSpinBox->value()));
-    xmlShading.appendChild(xmlAmbiant);
+    QDomElement xmlambient = doc.createElement("ambient");
+    xmlambient.setAttribute("value", shadeTmp.number(ui.ambientSpinBox->value()));
+    xmlShading.appendChild(xmlambient);
 
     QDomElement xmlDiffuse = doc.createElement("diffuse");
     xmlDiffuse.setAttribute("value", shadeTmp.number(ui.diffuseSpinBox->value()));
@@ -453,8 +481,8 @@ void VolumeRenderingWidget::save() {
     xmlSpecular.setAttribute("value", shadeTmp.number(ui.specularSpinBox->value()));
     xmlShading.appendChild(xmlSpecular);
 
-
     QFile file(filename);
+
     if (file.open(QFile::WriteOnly)) {
         QTextStream fileStream(&file);
         doc.save(fileStream, 2);
@@ -467,27 +495,23 @@ void VolumeRenderingWidget::save() {
 
 }
 
-// ---------------------- refreshVolumeRendering ----------------------
-void VolumeRenderingWidget::refreshVolumeRendering() {
-    auto* dadVR = dynamic_cast<VolumeRenderingAction*>(dad);
-    if (dadVR) {
-        dadVR->createVolumeRendering(myImage);
-    }
-
+// ---------------------- resetColorMap ----------------------
+void VolumeRenderingWidget::resetColorMap() {
+    load(myAction->property("Color Map File").toString());
 }
 
-// ---------------------- ambiantLightSliderChanged ----------------------
-void VolumeRenderingWidget::ambiantLightSliderChanged(int value) {
+// ---------------------- ambientLightSliderChanged ----------------------
+void VolumeRenderingWidget::ambientLightSliderChanged(int value) {
     double val = value / 100.0;
-    ui.ambiantSpinBox->setValue(val);
+    ui.ambientSpinBox->setValue(val);
 }
 
-// ---------------------- ambiantLightSpinBoxChanged ----------------------
-void VolumeRenderingWidget::ambiantLightSpinBoxChanged(double value) {
+// ---------------------- ambientLightSpinBoxChanged ----------------------
+void VolumeRenderingWidget::ambientLightSpinBoxChanged(double value) {
     int val = (int)(value * 100);
-    ui.ambiantSlider->blockSignals(true);
-    ui.ambiantSlider->setValue(val);
-    ui.ambiantSlider->blockSignals(false);
+    ui.ambientSlider->blockSignals(true);
+    ui.ambientSlider->setValue(val);
+    ui.ambientSlider->blockSignals(false);
 }
 
 // ---------------------- diffuseLightSliderChanged ----------------------
