@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -29,15 +29,16 @@
 // -- Core stuff
 #include <Application.h>
 #include <ExtensionManager.h>
-#include <MedicalImageViewer.h>
-#include <ActionViewer.h>
 #include <SettingsDialog.h>
 #include <Component.h>
-#include <Action.h>
-#include <Core.h> // for getActionDirectories()
+#include <MedicalImageViewer.h>
 #include <Explorer.h>
 #include <FrameExplorer.h>
 #include <PropertyExplorer.h>
+#include <ActionViewer.h>
+#include <Action.h>
+#include <Viewer.h>
+#include <Core.h> // for getActionDirectories()
 #include <Property.h>
 #include <PropertyObject.h>
 #include <Log.h>
@@ -54,6 +55,7 @@ using namespace camitk;
 #include <QMenuBar>
 #include <QStatusBar>
 #include <QDockWidget>
+#include <QStackedWidget>
 #include <QTranslator>
 
 // ------------- constructor -----------------
@@ -61,6 +63,8 @@ ImpMainWindow::ImpMainWindow() : MainWindow("imp") {
     // set the specific actions state machine icon
     setWindowIcon(QPixmap(":/applicationIcon"));
 
+    fileOpenDataDirectoryMenu = nullptr;
+
     // init all other GUI
     initActions();
     initMenuBar();
@@ -68,25 +72,64 @@ ImpMainWindow::ImpMainWindow() : MainWindow("imp") {
 
     // initialize architecture
     updateActionStates();
-    updateOpenDirectoryMenu();
 
     // now add the different viewers
-    setCentralViewer(MedicalImageViewer::getInstance());
-    addDockViewer(Qt::LeftDockWidgetArea, Explorer::getInstance());
-    addDockViewer(Qt::LeftDockWidgetArea, FrameExplorer::getInstance());
-    addDockViewer(Qt::LeftDockWidgetArea, PropertyExplorer::getInstance());
-    addDockViewer(Qt::RightDockWidgetArea, ActionViewer::getInstance());
+    MedicalImageViewer* medicalImageViewer = dynamic_cast<MedicalImageViewer*>(Application::getViewer("Medical Image Viewer"));
 
-    //Merge Explorer and FrameExplorer viewers in one layout
-    QDockWidget* dockWidgetExplorer = dockWidgetMap.value(Explorer::getInstance(), NULL);
-    QDockWidget* dockWidgetFrameExplorer = dockWidgetMap.value(FrameExplorer::getInstance(), NULL);
-    if (dockWidgetExplorer && dockWidgetFrameExplorer) {
+    if (medicalImageViewer != nullptr) {
+        setCentralViewer(medicalImageViewer);
+    }
+    else {
+        CAMITK_ERROR(tr("Cannot find \"Medical Image Viewer\". This viewer is mandatory for running camitk-imp."))
+    }
+
+    Explorer* explorer = dynamic_cast<Explorer*>(Application::getViewer("Explorer"));
+
+    if (explorer != nullptr) {
+        addDockViewer(Qt::LeftDockWidgetArea, explorer);
+    }
+    else {
+        CAMITK_ERROR(tr("Cannot find \"Explorer\" viewer. This viewer is mandatory for running camitk-imp."))
+    }
+
+    FrameExplorer* frameExplorer = dynamic_cast<FrameExplorer*>(Application::getViewer("Frame Explorer"));
+
+    if (frameExplorer != nullptr) {
+        addDockViewer(Qt::LeftDockWidgetArea, frameExplorer);
+    }
+    else {
+        CAMITK_ERROR(tr("Cannot find \"Frame Explorer\" viewer. This viewer is mandatory for running camitk-imp."))
+    }
+
+    PropertyExplorer* propertyExplorer = dynamic_cast<PropertyExplorer*>(Application::getViewer("Property Explorer"));
+
+    if (frameExplorer != nullptr) {
+        addDockViewer(Qt::LeftDockWidgetArea, propertyExplorer);
+    }
+    else {
+        CAMITK_ERROR(tr("Cannot find \"Property Explorer\" viewer. This viewer is mandatory for running camitk-imp."))
+    }
+
+    ActionViewer* actionViewer = dynamic_cast<ActionViewer*>(Application::getViewer("Action Viewer"));
+
+    if (actionViewer != nullptr) {
+        addDockViewer(Qt::RightDockWidgetArea, actionViewer);
+        actionViewer->setSearchPanelVisible(true);
+        showDockViewer(actionViewer, false);
+    }
+    else {
+        CAMITK_ERROR(tr("Cannot find \"Action Viewer\". This viewer is mandatory for running camitk-imp."))
+    }
+
+    // Merge Explorer and FrameExplorer viewers in one layout
+    QDockWidget* dockWidgetExplorer = dockWidgetMap.value(explorer, nullptr);
+    QDockWidget* dockWidgetFrameExplorer = dockWidgetMap.value(frameExplorer, nullptr);
+
+    if (dockWidgetExplorer != nullptr && dockWidgetFrameExplorer != nullptr) {
         tabifyDockWidget(dockWidgetExplorer, dockWidgetFrameExplorer);
         dockWidgetExplorer->raise();
     }
 
-    showViewer(ActionViewer::getInstance(), false);
-    ActionViewer::getInstance()->setSearchPanelVisible(true);
     showStatusBar(true);
 }
 
@@ -111,10 +154,9 @@ void ImpMainWindow::aboutToShow() {
 void ImpMainWindow::refresh() {
     MainWindow::refresh();
 
-    // update Data directory menu
-    updateOpenDirectoryMenu();
     // update all the action states
     updateActionStates();
+
     // update menu
     updateRecentDocumentsMenu();
 }
@@ -135,18 +177,30 @@ void ImpMainWindow::setCentralViewer(Viewer* theViewer) {
 
 // ------------- initActions -----------------
 void ImpMainWindow::initActions() {
-    //-- Check for action applications
-    ActionExtension* applicationActionExtension = NULL;
+    //-- Check for applications level action (file, edit and help)
+    QStringList applicationLevelNames = {"File Application Level Actions", "Edit Application Level Actions", "Help Application Level Actions"};
+    QMap<QString, ActionExtension*> applicationLevelActionExtensions;
     QListIterator<ActionExtension*> it(ExtensionManager::getActionExtensionsList());
-    while (it.hasNext() && applicationActionExtension == NULL) {
+
+    while (it.hasNext() && applicationLevelActionExtensions.size() < 3) {
         ActionExtension* current = it.next();
-        if (current->getName() == "Application Level Actions") {
-            applicationActionExtension = current;
+
+        int i = 0;
+        while (i < applicationLevelNames.size() && current->getName() != applicationLevelNames.at(i)) {
+            i++;
+        }
+        if (i < applicationLevelNames.size()) {
+            applicationLevelActionExtensions.insert(applicationLevelNames.at(i), current);
         }
     }
-    if (applicationActionExtension == NULL) {
-        CAMITK_ERROR(tr("Application error: the \"Application Level Actions\" extension is required by this application but could not be found\nIn any of the following extension directories:\n - %1\nPlease check your CamiTK configuration and/or installation using \"camitk-config --config\"").arg(Core::getActionDirectories().join("\n - ")))
-        exit(EXIT_FAILURE);
+
+    if (applicationLevelActionExtensions.size() < 3) {
+        for(QString name : applicationLevelNames) {
+            if (!applicationLevelActionExtensions.contains(name)) {
+                CAMITK_ERROR(tr("Application error: the \"%1\" action extension is required by camitk-imp but could not be found\nIn any of the following extension directories:\n - %2\nPlease check your CamiTK configuration and/or installation using \"camitk-config --config\"").arg(name, Core::getActionDirectories().join("\n - ")))
+                exit(EXIT_FAILURE);
+            }
+        }
     }
 
     //--- actions of the File menu
@@ -195,7 +249,7 @@ void ImpMainWindow::initActions() {
     // global shortcut, NOTE: the action should also be added as an ImpMainWindow action, otherwise the shortcut will not work when
     // the menu bar is hidden! (see initMenuBar() method)
     viewMenuBar->setShortcutContext(Qt::ApplicationShortcut);
-    connect(viewMenuBar, SIGNAL(triggered(bool)), this, SLOT(showMenuBar(bool)));
+    connect(viewMenuBar, SIGNAL(triggered()), this, SLOT(toggleMenuBar()));
     // NOTE: viewMenuBar can be used to hide the menu bar, if the menu bar is not visible,
     // since Qt4, it does not receive any event (and thus there is no way to set the menu
     // visible again! Which is quite annoying).
@@ -222,17 +276,13 @@ void ImpMainWindow::initActions() {
 
 }
 
-//**************************************************************************
-
+// ------------- initMenuBar -----------------
 void ImpMainWindow::initMenuBar() {
     // -- file
     fileMenu = new QMenu(tr("&File"));
     fileMenu->addAction(fileOpen);
 
-    fileOpenDataDirectoryMenu = new QMenu(tr("Open &Data Directory..."));
-    fileOpenDataDirectoryMenu->setIcon(QPixmap(":/fileOpen"));
-    fileOpenDataDirectoryMenu->setEnabled(false);
-    fileMenu->addMenu(fileOpenDataDirectoryMenu);
+    updateOpenDirectoryMenu();
 
     fileMenu->addAction(fileClose);
     fileMenu->addAction(fileCloseAll);
@@ -272,6 +322,7 @@ void ImpMainWindow::initMenuBar() {
     this->addAction(whatsThisAction);
     helpMenu->addSeparator();
     helpMenu->addAction(helpShowConsole);
+    helpMenu->addAction(Application::getAction("Logger Parameters")->getQAction());
     helpMenu->addSeparator();
     helpMenu->addAction(changeLanguage);
     helpMenu->addSeparator();
@@ -292,9 +343,9 @@ void ImpMainWindow::initMenuBar() {
 
 //--------------------------initToolBar-------------------------------
 void ImpMainWindow::initToolBar() {
-    mainToolbar = addToolBar("Main toolbar");
+    mainToolbar = addToolBar("Main Toolbar");
     // ensure object name is set for saving the state
-    mainToolbar->setObjectName("imp main toolbar");
+    mainToolbar->setObjectName("ImpMainWindow Main Toolbar");
     mainToolbar->addAction(fileOpen);
     mainToolbar->addAction(fileSave);
     mainToolbar->addAction(QWhatsThis::createAction(this));
@@ -336,6 +387,7 @@ void ImpMainWindow::updateActionStates() {
     else {
         // add all generic actions
         ActionList allActions = Application::getActions(NULL);
+
         foreach (Action* action, allActions) {
             actionMenu->addAction(action->getQAction());
             actionMenu->setEnabled(true);
@@ -356,22 +408,19 @@ void ImpMainWindow::showToolbar(bool b) {
     mainToolbar->setVisible(b);
 }
 
-// ------------- showMenuBar -----------------
-void ImpMainWindow::showMenuBar(bool b) {
-    if (!b) {
+// ------------- toggleMenuBar -----------------
+void ImpMainWindow::toggleMenuBar() {
+    bool menuBarVisible = menuBar()->isVisible();
+
+    if (menuBarVisible) {
         // warn the user first
         CAMITK_WARNING(tr("Hide menu: the menu is going to be hidden. There is only one way to make it reappear: you need to press CTRL+M again."))
-        viewMenuBar->blockSignals(true);
-        viewMenuBar->setChecked(true);
-        viewMenuBar->blockSignals(false);
-    }
-    else {
-        menuBar()->setVisible(b);
-        viewMenuBar->blockSignals(true);
-        viewMenuBar->setChecked(b);
-        viewMenuBar->blockSignals(false);
-        showStatusBar(b);
     }
+
+    menuBar()->setVisible(!menuBarVisible);
+    viewMenuBar->blockSignals(true);
+    viewMenuBar->setChecked(!menuBarVisible);
+    viewMenuBar->blockSignals(false);
 }
 
 // ------------- showStatusBar -----------------
@@ -381,38 +430,46 @@ void ImpMainWindow::showStatusBar(bool b) {
 
 // ------------------------ resetWindows ----------------------------
 void ImpMainWindow::resetWindows() {
-    for (QMap<Viewer*, QDockWidget*>::iterator it = dockWidgetMap.begin(); it != dockWidgetMap.end(); it++) {
-        if (it.value()->widget() == Explorer::getInstance()->getWidget(NULL)) {
+    for (QMap<Viewer*, QDockWidget*>::iterator it = dockWidgetMap.begin(); it != dockWidgetMap.end(); ++it) {
+        if (it.key()->getName() == "Explorer") {
             removeDockWidget(it.value());
             addDockWidget(Qt::LeftDockWidgetArea, it.value());
             it.value()->show();
         }
-        else if (it.value()->widget() == FrameExplorer::getInstance()->getWidget(NULL)) {
-            removeDockWidget(it.value());
-            addDockWidget(Qt::LeftDockWidgetArea, it.value());
-            it.value()->show();
-        }
-        else if (it.value()->widget() == PropertyExplorer::getInstance()->getWidget()) {
-            removeDockWidget(it.value());
-            addDockWidget(Qt::LeftDockWidgetArea, it.value());
-            it.value()->show();
-        }
-        else if (it.value()->widget() == ActionViewer::getInstance()->getWidget()) {
-            removeDockWidget(it.value());
-            addDockWidget(Qt::RightDockWidgetArea, it.value());
-            it.value()->show();
+        else {
+            if (it.key()->getName() == "Frame Explorer") {
+                removeDockWidget(it.value());
+                addDockWidget(Qt::LeftDockWidgetArea, it.value());
+                it.value()->show();
+            }
+            else {
+                if (it.key()->getName() == "Property Explorer") {
+                    removeDockWidget(it.value());
+                    addDockWidget(Qt::LeftDockWidgetArea, it.value());
+                    it.value()->show();
+                }
+                else {
+                    if (it.key()->getName() == "Action Viewer") {
+                        removeDockWidget(it.value());
+                        addDockWidget(Qt::RightDockWidgetArea, it.value());
+                        it.value()->show();
+                    }
+                }
+            }
         }
     }
-    //Merge Explorer and FrameExplorer viewers in one layout
-    QDockWidget* dockWidgetExplorer = dockWidgetMap.value(Explorer::getInstance(), NULL);
-    QDockWidget* dockWidgetFrameExplorer = dockWidgetMap.value(FrameExplorer::getInstance(), NULL);
+
+    // Merge Explorer and FrameExplorer viewers in one layout
+    QDockWidget* dockWidgetExplorer = dockWidgetMap.value(Application::getViewer("Explorer"), NULL);
+    QDockWidget* dockWidgetFrameExplorer = dockWidgetMap.value(Application::getViewer("Frame Explorer"), NULL);
+
     if (dockWidgetExplorer && dockWidgetFrameExplorer) {
         tabifyDockWidget(dockWidgetExplorer, dockWidgetFrameExplorer);
         dockWidgetExplorer->raise();
     }
 
     showStatusBar(true);
-    showMenuBar(true);
+    menuBar()->setVisible(true);
     showToolbar(true);
 
     // reset geometry to default
@@ -420,32 +477,9 @@ void ImpMainWindow::resetWindows() {
     move(0, 0);
 }
 
-// ------------------------ updateDataDirectoryMenu ----------------------------
-void ImpMainWindow::updateOpenDirectoryMenu() {
-    unsigned int nrOfDataDirectoryManager = 0;
-
-    disconnect(fileOpenDataDirectoryMenu, SIGNAL(triggered(QAction*)), this, SLOT(openDirectory(QAction*)));
-    fileOpenDataDirectoryMenu->clear();
-    fileOpenDataDirectoryMenu->setEnabled(false);
-
-    foreach (QString name, ExtensionManager::getDataDirectoryExtNames()) {
-        nrOfDataDirectoryManager++;
-        QAction* openDirectory = new QAction(name, this);
-        openDirectory->setStatusTip(tr(QString("Opens data directory for " + name).toStdString().c_str()));
-        openDirectory->setWhatsThis(tr(QString("Opens data directory for " + name).toStdString().c_str()));
-
-        fileOpenDataDirectoryMenu->addAction(openDirectory);
-    }
-
-    if (nrOfDataDirectoryManager > 0) {
-        fileOpenDataDirectoryMenu->setEnabled(true);
-        connect(fileOpenDataDirectoryMenu, SIGNAL(triggered(QAction*)), this, SLOT(openDirectory(QAction*)));
-    }
-}
-
 // ------------- openDataDirectory -----------------
-void ImpMainWindow::openDirectory(QAction* emitter) {
-    QString pluginName = emitter->text().replace("&", "");
+void ImpMainWindow::openDataDirectory(QString plugin) {
+    QString pluginName = plugin;
 
     statusBar()->showMessage(tr(QString("Opening " + pluginName + " directory...").toStdString().c_str()));
 
@@ -456,7 +490,7 @@ void ImpMainWindow::openDirectory(QAction* emitter) {
         // set waiting cursor
         QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
-        // Instanciate a Component to represent the data contained in the directory
+        // instantiate a Component to represent the data contained in the directory
         Component* comp = Application::openDirectory(dir, pluginName);
 
         if (!comp) {
@@ -478,6 +512,9 @@ void ImpMainWindow::openDirectory(QAction* emitter) {
 
 // ------------- updateViewMenu -----------------
 void ImpMainWindow::updateViewMenu() {
+    // From Qt documentation:
+    // "Removes all the menu's actions. Actions owned by the menu and not shown in any other widget are deleted"
+    // this will therefore automatically delete the toggle central viewer's action
     viewMenu->clear();
 
     // insert viewers menu
@@ -489,19 +526,67 @@ void ImpMainWindow::updateViewMenu() {
         }
     }
 
-    // insert viewers on/off actions
-    viewMenu->addSeparator()->setText(tr("Toggle Viewers"));
-    viewMenu->addAction(viewResetWindows);
+    viewMenu->addSeparator();
 
-    for (QMap<Viewer*, QDockWidget*>::iterator it = dockWidgetMap.begin(); it != dockWidgetMap.end(); it++) {
-        viewMenu->addAction(it.value()->toggleViewAction());
+    QMenu* toggleCentralWidgetMenu = viewMenu->addMenu(tr("Toggle Central Viewer"));
+
+    // add other viewers that are not initially part of imp, but that are
+    // added by supplementary viewer extensions
+    foreach (Viewer* viewer, Application::getViewers()) {
+        // only control the embedded viewer that are not already embedded somewhere or is already in the centralViewer stack
+        if (viewer->getType() == Viewer::EMBEDDED && (viewer->getEmbedder() == centralWidget()->layout() || viewer->getEmbedder() == nullptr)) {
+            // viewerAction is own by viewMenu, viewMenu->clear() will delete it
+            QAction* viewerAction = toggleCentralWidgetMenu->addAction(viewer->getName());
+            viewerAction->setCheckable(true);
+            viewerAction->setChecked(viewer == centralViewer);
+            viewerAction->setIcon(viewer->getIcon());
+            QString tipString = "Hide/show the " + viewer->getName() + " in the central viewer";
+            viewerAction->setStatusTip(tr(tipString.toStdString().c_str()));
+            viewerAction->setWhatsThis(tr(tipString.toStdString().c_str()));
+            // add the toggle action slot using C++11 lambda so that everything is contained inside viewMenu
+            connect(viewerAction, &QAction::toggled, this, [ this, viewer ](bool) {
+                setCentralViewer(viewer);
+            });
+        }
+    }
+
+    // insert viewers on/off actions
+    QMenu* toggleDockedMenu = viewMenu->addMenu(tr("Toggle Docked Viewers"));
+
+    toggleDockedMenu->addAction(viewResetWindows);
+
+    foreach (Viewer* viewer, Application::getViewers()) {
+        // add the
+        if (viewer->getType() == Viewer::DOCKED) {
+            QDockWidget* dockWidget = dockWidgetMap.value(viewer);
+
+            if (dockWidget != nullptr) {
+                dockWidget->toggleViewAction()->setIcon(viewer->getIcon());
+                toggleDockedMenu->addAction(dockWidget->toggleViewAction());
+            }
+            else {
+                // add a specific action to add this viewer in the right dock area
+                // viewerAction is own by viewMenu, viewMenu->clear() will delete it
+                QAction* viewerDockAction = toggleDockedMenu->addAction(viewer->getName());
+                viewerDockAction->setCheckable(true);
+                viewerDockAction->setChecked(false);
+                viewerDockAction->setIcon(viewer->getIcon());
+                QString tipString = "Hide/show the " + viewer->getName() + " inside a dock";
+                viewerDockAction->setStatusTip(tr(tipString.toStdString().c_str()));
+                viewerDockAction->setWhatsThis(tr(tipString.toStdString().c_str()));
+                // add the toggle action slot using C++11 lambda so that everything is contained inside viewMenu
+                connect(viewerDockAction, &QAction::toggled, this, [ this, viewer ](bool) {
+                    addDockViewer(Qt::RightDockWidgetArea, viewer);
+                });
+            }
+        }
     }
 
     // insert generic on/off actions
-    viewMenu->addSeparator()->setText(tr("Other Toggle"));
-    viewMenu->addAction(viewMenuBar);
-    viewMenu->addAction(viewStatusBar);
-    viewMenu->addAction(mainToolbar->toggleViewAction());
+    QMenu* otherToggleMenu = viewMenu->addMenu(tr("Other Toggle"));
+    otherToggleMenu->addAction(viewMenuBar);
+    otherToggleMenu->addAction(viewStatusBar);
+    otherToggleMenu->addAction(mainToolbar->toggleViewAction());
 }
 
 // ------------------------------ slotEditSettings -------------------------------
@@ -531,7 +616,7 @@ void ImpMainWindow::editSettings() {
     }
 
     // check if there are anything to change in the data directory menu (for data directory manager)
-    updateOpenDirectoryMenu();
+    ImpMainWindow::updateOpenDirectoryMenu();
 }
 
 // ------------- setApplicationConsole -----------------
@@ -547,7 +632,7 @@ void ImpMainWindow::openRecentDocuments() {
 
     if (action) {
         // check if this recent document still exists!
-        if (QFileInfo(action->data().toString()).exists()) {
+        if (QFileInfo::exists(action->data().toString())) {
             Action* openFileAction = Application::getAction("Open File");
             openFileAction->setProperty("File Name", action->data().toString()); // avoid opening a dialog, as the file path is already know
             openFileAction->applyAndRegister();
@@ -585,7 +670,72 @@ void ImpMainWindow::saveHistoryAsSCXML() {
     Application::saveHistoryAsSXML();
 }
 
+// ------------------------- updateOpenDirectoryMenu ---------------------------
+void ImpMainWindow::updateOpenDirectoryMenu() {
+    QStringList dirExt = ExtensionManager::getDataDirectoryExtNames();
 
+    if (dirExt.size() > 1) {
+        if (fileOpenDataDirectoryMenu == nullptr) {
+            fileOpenDataDirectoryMenu = new QMenu("Open Data Directory");
+            fileMenu->addMenu(fileOpenDataDirectoryMenu);
+        }
+        else {
+            disconnect(fileOpenDataDirectoryMenu, 0, 0, 0);
+            fileOpenDataDirectoryMenu->clear();
+        }
 
+        fileOpenDataDirectoryMenu->setIcon(QPixmap(":/fileOpen"));
 
+        for (auto& dirExtName : qAsConst(dirExt)) {
+            QAction* openDirectory = new QAction(dirExtName, this);
+            openDirectory->setStatusTip(tr(QString("Opens data directory for " + dirExtName).toStdString().c_str()));
+            openDirectory->setWhatsThis(tr(QString("Opens data directory for " + dirExtName).toStdString().c_str()));
+            connect(openDirectory, &QAction::triggered, this, [ this, dirExtName ]() {
+                this->openDataDirectory(dirExtName);
+            });
 
+            fileOpenDataDirectoryMenu->addAction(openDirectory);
+
+        }
+    }
+    else if (dirExt.size() == 1) {
+        // if it exists, remove the default directory extension menu (that has only one submenu)
+        // from the file "open" action list
+        bool updateAction = false;
+        QList<QAction*> menuActions = fileMenu->actions();
+        int indexAction = 0;
+
+        while (indexAction < menuActions.size() && !updateAction) {
+            if (menuActions.at(indexAction)->text().contains(" Directory")) {
+                emit menuActions.at(indexAction)->destroyed();
+                updateAction = true;
+            }
+
+            indexAction++;
+        }
+
+        // create the single open directory action
+        QString dirExtName = dirExt.at(0);
+        QAction* openDirectoryAction = new QAction("Open " + dirExtName + " Directory", this);
+
+        if (!updateAction) {
+            // there was no specific menu, just add the single "open directory" action in the file menu
+            fileMenu->addAction(openDirectoryAction);
+        }
+
+        openDirectoryAction->setStatusTip(tr(QString("Opens data directory for " + dirExtName).toStdString().c_str()));
+        openDirectoryAction->setWhatsThis(tr(QString("Opens data directory for " + dirExtName).toStdString().c_str()));
+
+        connect(openDirectoryAction, &QAction::triggered, this, [ this, dirExtName ]() {
+            this->openDataDirectory(dirExtName);
+        });
+
+        openDirectoryAction->setIcon(QPixmap(":/fileOpen"));
+    }
+    else {
+        fileOpenDataDirectoryMenu = new QMenu("No plugins loaded to Open Data Directory");
+        fileOpenDataDirectoryMenu->setIcon(QPixmap(":/fileOpen"));
+        fileOpenDataDirectoryMenu->setEnabled(false);
+        fileMenu->addMenu(fileOpenDataDirectoryMenu);
+    }
+}
