@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -81,24 +81,24 @@ QString ActionGenerator::getComponentNamespace() {
 
 // -------------------- createFromDom --------------------
 void ActionGenerator::createFromDom(Action& dom) {
-    std::cout << "Creating action..." << std::endl;
-    this->name          = QString(dom.name().c_str());
-    this->description   = QString(dom.description().c_str()).simplified();
-    this->className     = ClassNameHandler::getClassName(this->name);
-    this->componentName = QString(dom.component().c_str());
+    name          = QString(dom.name().c_str());
+    description   = QString(dom.description().c_str()).simplified();
+    className     = ClassNameHandler::getClassName(name);
+    componentName = QString(dom.component().c_str());
+    std::cout << "Generating Action \"" << className.toStdString() << "\"..." << std::endl;
 
     // classification
     // 1- Family
-    this->family           = QString(dom.classification().family().c_str());
+    family           = QString(dom.classification().family().c_str());
     // 2- Checking tags
     for (Classification::tag_const_iterator it = dom.classification().tag().begin(); it != dom.classification().tag().end(); it++) {
         tags << QString((*it).c_str());
     }
     // 3- Checking  itk flag
-    this->isItkFilter = false;
+    isItkFilter = false;
     if (dom.classification().itkFilter().present()) {
-        this->isItkFilter = true;
-        this->itkFilterOutputType = QString(dom.classification().itkFilter().get().outputType().c_str());
+        isItkFilter = true;
+        itkFilterOutputType = QString(dom.classification().itkFilter().get().outputType().c_str());
     }
 
     // parameters
@@ -113,7 +113,7 @@ void ActionGenerator::createFromDom(Action& dom) {
 }
 
 QString ActionGenerator::getClassName() const {
-    return this->className;
+    return className;
 }
 
 
@@ -140,7 +140,7 @@ void ActionGenerator::writeHFile(QString directoryName) {
         QString msg = "Exception from extension generation: \n    Cannot write on file " + extFilePath.fileName() + "\n";
         throw (msg);
     }
-    std::cout << "generating " << extFilePath.absolutePath().toStdString() << "/" << className.toStdString() << ".h" << std::endl;
+
     QTextStream out(&extFile);
     QString text;
     do {
@@ -228,7 +228,7 @@ void ActionGenerator::writeCFile(QString directoryName) {
     // Additional includes needed by parameter types
     QSet<QString> additionalIncludes;
     // here we use a QSet to avoid the repetion of includes
-    for (auto actionParameter : actionParameters) {
+    for (auto actionParameter : qAsConst(actionParameters)) {
         if (actionParameter->needsAdditionalInclude()) {
             additionalIncludes.insert(actionParameter->getAdditionalInclude());
         }
@@ -270,7 +270,7 @@ void ActionGenerator::writeCFile(QString directoryName) {
         text.replace(QRegExp("@ACTION_DESCRIPTION@"), description);
         text.replace(QRegExp("@FAMILY@"), family);
         if (text.contains(QRegExp("@ADDITIONAL_INCLUDES@"))) {
-            for (QSet<QString>::const_iterator incl = additionalIncludes.begin(); incl != additionalIncludes.end(); incl++) {
+            for (QSet<QString>::const_iterator incl = additionalIncludes.constBegin(); incl != additionalIncludes.constEnd(); ++incl) {
                 out << (*incl) << endl;
             }
         }
@@ -386,7 +386,7 @@ void ActionGenerator::writeCFile(QString directoryName) {
                 QStringList addTheProps;
                 while (! text.contains(QRegExp("@END_ADDPROPERTIES@")) &&
                         ! text.contains(QRegExp("@ELSE_ADDPROPERTIES@"))) {
-                    for (auto param : actionParameters) {
+                    for (auto param : qAsConst(actionParameters)) {
                         QString textTmp = text;
 
                         textTmp.replace(QRegExp("@PROPERTY_TYPE@"), param->getType());
