@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -22,11 +22,13 @@
  *
  * $CAMITK_LICENCE_END$
  ****************************************************************************/
+
 #include "CepGenerator.h"
 #include "ExtensionGenerator.h"
 #include "ActionExtensionGenerator.h"
 #include "ComponentExtensionGenerator.h"
 #include "LibraryGenerator.h"
+#include "ViewerExtensionGenerator.h"
 #include "ClassNameHandler.h"
 
 // includes from std
@@ -40,13 +42,15 @@
 #include <Cep.hxx>
 #include <Contact.hxx>
 
+
 static const QString cepActionsDirectoryName          = "actions";
-static const QString cepApplicationsDirectoryName  = "applications";
-static const QString cepComponentsDirectoryName = "components";
+static const QString cepApplicationsDirectoryName     = "applications";
+static const QString cepComponentsDirectoryName       = "components";
 static const QString cepLibrariesDirectoryName        = "libraries";
-static const QString cepManifestName = "CEPContent";
-static const QString cepManifestExtension = ".xml";
-static const QString cepSchemaNamespace = "http://camitk.imag.fr/cepcoreschema";
+static const QString cepViewersDirectoryName          = "viewers";
+static const QString cepManifestName                  = "CEPContent";
+static const QString cepManifestExtension             = ".xml";
+static const QString cepSchemaNamespace               = "http://camitk.imag.fr/cepcoreschema";
 
 inline void initMyResource() {
     Q_INIT_RESOURCE(cepgenerator);
@@ -72,7 +76,6 @@ CepGenerator::CepGenerator(std::unique_ptr< cepcoreschema::Cep > domCep, QString
 }
 
 CepGenerator::~CepGenerator() {
-    std::cout << "inside CepGenerator destructor..." << std::endl;
     for (auto& extension : extensions) {
         delete extension;
     }
@@ -100,26 +103,26 @@ void CepGenerator::setDevDirectoryName(QString devDirectoryName) {
 }
 
 void CepGenerator::createDomTree() {
-    std::cout << "Creating Dom Tree..." << std::endl;
+    std::cout << "Parsing CEP XML..." << std::endl;
     std::string xmlFileStr = xmlFileName.canonicalFilePath().toStdString();
 
     try {
-        this->domCep = cep(xmlFileStr, xml_schema::flags::dont_validate);
+        domCep = cep(xmlFileStr, xml_schema::flags::dont_validate);
     }
     catch (const xml_schema::exception& e) {
-        std::cout << "Parsing exception during CEP file generation in XML document " << xmlFileStr << ": " << std::endl;
-        std::cout << e << std::endl;
+        std::cerr << "Parsing exception during in CEP XML document " << xmlFileStr << ": " << std::endl;
+        std::cerr << e << std::endl;
         throw std::invalid_argument(e.what());
     }
 }
 
 void CepGenerator::generateDirectoryTree() {
-    std::cout << "Generating Directory Tree..." << std::endl;
+    std::cout << "Generating source directory tree..." << std::endl;
 
     // --------------------   Create a sub-directory with the CEP name
     QString cepName(domCep->name().c_str());
     // A directory name should be low case with no space
-    this->cepDirectoryName = ClassNameHandler::getDirectoryName(cepName);
+    cepDirectoryName = ClassNameHandler::getDirectoryName(cepName);
 
     QDir currentDirectory;
     currentDirectory.cd(devDirectoryName.absolutePath());
@@ -146,8 +149,10 @@ void CepGenerator::generateDirectoryTree() {
     generateComponentsDirectory();
     // Libraries
     generateLibrariesDirectory();
+    // Viewers
+    generateViewersDirectory();
 
-    // Create build and doc directories ???
+    // Create doc directories ???
 
 }
 
@@ -203,6 +208,7 @@ void CepGenerator::generateGlobalCMakeLists() {
     // Setting CMakeLists parameters
     // project name
     QString projectNameString(domCep->name().c_str());
+
     // contacts
     QString contactString = "";
     Contact contact = domCep->contact();
@@ -237,6 +243,7 @@ void CepGenerator::generateGlobalCMakeLists() {
     QString text;
     do {
         text = in.readLine();
+        text.replace(QRegExp("@CMAKE_PROJECT_NAME@"), cepDirectoryName);
         text.replace(QRegExp("@PROJECT_NAME@"), projectNameString);
         text.replace(QRegExp("@CONTACT@"), contactString);
         text.replace(QRegExp("@DESCRIPTION@"), descriptionString);
@@ -251,20 +258,20 @@ void CepGenerator::generateGlobalCMakeLists() {
 void CepGenerator::setLicence() {
     // Get the copyright in the CEP DOM...
     if (domCep->copyright().present()) {
-        std::cout << "I found a licence ! " << std::endl;
-        this->licence = QString(domCep->copyright().get().c_str());
+        std::cout << "Licence found." << std::endl;
+        licence = QString(domCep->copyright().get().c_str());
     }
     else {
-        std::cout << "No licence found... Copying the default one..." << endl;
+        std::cout << "No licence found: using default license." << std::endl;
         QFile defaultLicenceFile(":/resources/COPYRIGHT.in");
         defaultLicenceFile.open(QIODevice::ReadOnly | QIODevice::Text);
         QTextStream in(&defaultLicenceFile);
         QString text;
 
-        this->licence = "";
+        licence = "";
         text = in.readLine();
         while (! text.isNull()) {
-            this->licence += text + "\n";
+            licence += text + "\n";
             text = in.readLine();
         }
     }
@@ -285,7 +292,7 @@ void CepGenerator::generateCopyright() {
     }
     QTextStream out(&copyrightOutFile);
 
-    QTextStream in(&(this->licence));
+    QTextStream in(&(licence));
     QString text;
     do {
         text = in.readLine();
@@ -333,7 +340,6 @@ void CepGenerator::generateReadme() {
 }
 
 void CepGenerator::copyFindCamiTK() {
-
     // Setting the CEP root directory where the manifest should be
     QDir currentDirectory;
     currentDirectory.cd(devDirectoryName.absolutePath());
@@ -354,6 +360,7 @@ void CepGenerator::generateActionsDirectory() {
     cmakelistFileName = currentDirectory.absolutePath() + "/CMakeLists.txt";
     QFile::copy(":/resources/actions.CMakeLists.txt.in", cmakelistFileName);
 }
+
 void CepGenerator::generateApplicationsDirectory() {
     QString cmakelistFileName;
     QDir currentDirectory;
@@ -391,10 +398,20 @@ void CepGenerator::generateLibrariesDirectory() {
     QFile::copy(":/resources/libraries.CMakeLists.txt.in", cmakelistFileName);
 }
 
+void CepGenerator::generateViewersDirectory() {
+    QString cmakelistFileName;
+    QDir currentDirectory;
+    currentDirectory.cd(devDirectoryName.absolutePath());
+    currentDirectory.cd(cepDirectoryName);
+    // createing actions directory
+    currentDirectory.mkdir(cepViewersDirectoryName);
+    // copying CMakeLists.txt
+    currentDirectory.cd(cepViewersDirectoryName);
+    cmakelistFileName = currentDirectory.absolutePath() + "/CMakeLists.txt";
+    QFile::copy(":/resources/viewers.CMakeLists.txt.in", cmakelistFileName);
+}
 
 void CepGenerator::createActionExtensionDoms() {
-    std::cout << "creating action extension doms..." << std::endl;
-    QString cmakelistFileName;
     QDir currentDirectory;
     currentDirectory.cd(devDirectoryName.absolutePath());
     currentDirectory.cd(cepDirectoryName);
@@ -413,8 +430,6 @@ void CepGenerator::createActionExtensionDoms() {
 }
 
 void CepGenerator::createComponentExtensionDoms() {
-    std::cout << "creating component extension doms..." << std::endl;
-    QString cmakelistFileName;
     QDir currentDirectory;
     currentDirectory.cd(devDirectoryName.absolutePath());
     currentDirectory.cd(cepDirectoryName);
@@ -433,7 +448,6 @@ void CepGenerator::createComponentExtensionDoms() {
 }
 
 void CepGenerator::createLibrarieDoms() {
-    std::cout << "creating library doms..." << std::endl;
     QDir currentDirectory;
     currentDirectory.cd(devDirectoryName.absolutePath());
     currentDirectory.cd(cepDirectoryName);
@@ -446,20 +460,35 @@ void CepGenerator::createLibrarieDoms() {
             // Is it only possible ? http://stackoverflow.com/questions/12674735/function-wont-accept-iterator-to-auto-ptr
             Library& lib = (*i);
             LibraryGenerator* generator = new LibraryGenerator(lib, currentDirectory.absolutePath(), licence);
-            this->extensions.append(generator);
+            extensions.append(generator);
         }
     }
 
 }
 
-void CepGenerator::generateExtensions() {
-    std::cout << "inside generateExtension" << std::endl;
+void CepGenerator::createViewersExtensionDoms() {
+    QDir currentDirectory;
+    currentDirectory.cd(devDirectoryName.absolutePath());
+    currentDirectory.cd(cepDirectoryName);
+    currentDirectory.cd(cepViewersDirectoryName);
 
+    if (domCep->viewerExtensions().present()) {
+        ViewerExtensions extensions = domCep->viewerExtensions().get();
+        for (auto i = extensions.viewerExtension().begin(); i != extensions.viewerExtension().end(); i++) {
+            // TODO: find a way to use std:auto_ptr here for the dom extension instead of creating a new instance
+            // Is it only possible ? http://stackoverflow.com/questions/12674735/function-wont-accept-iterator-to-auto-ptr
+            ViewerExtension& extension = (*i);
+            ViewerExtensionGenerator* generator = new ViewerExtensionGenerator(extension, currentDirectory.absolutePath(), licence);
+            this->extensions.append(generator);
+        }
+    }
+}
+
+void CepGenerator::generateExtensions() {
     // Component and Action extensions genration
     for (auto& extension : extensions) {
         extension->generateExtension();
     }
-
 }
 
 void CepGenerator::process() {
@@ -468,5 +497,6 @@ void CepGenerator::process() {
     createComponentExtensionDoms();
     createActionExtensionDoms();
     createLibrarieDoms();
+    createViewersExtensionDoms();
     generateExtensions();
 }
