@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -28,6 +28,7 @@
 #include "Core.h"
 #include "Application.h"
 #include "Action.h"
+#include "Viewer.h"
 
 // -- QT stuff
 #include <QDir>
@@ -42,21 +43,29 @@ namespace camitk {
 
 // -------------------- autoload --------------------
 void ExtensionManager::autoload() {
+    autoload(VIEWER);
     autoload(COMPONENT);
     autoload(ACTION);
 }
 
 void ExtensionManager::autoload(ExtensionManager::ExtensionType type) {
     QStringList extensionDir;
+
     switch (type) {
         case ACTION:
             extensionDir = Core::getActionDirectories();
             break;
+
         case COMPONENT:
             extensionDir = Core::getComponentDirectories();
             break;
+
+        case VIEWER:
+            extensionDir = Core::getViewerDirectories();
+            break;
+
         default:
-            CAMITK_TRACE_ALT(tr("autoload of unknown type: type should be either ACTION or COMPONENT, other extensions are not implemented yet"))
+            CAMITK_TRACE_ALT(tr("autoload of unknown type: type should be either ACTION or COMPONENT or VIEWER, other extensions are not implemented yet"))
             break;
     }
 
@@ -64,11 +73,14 @@ void ExtensionManager::autoload(ExtensionManager::ExtensionType type) {
     QStringList extensionFileNames;
     QStringList extensionAbsoluteFileNames;
     QMap<QString, QDir> extensionUniqueDir;
+
     foreach (QString dirName, extensionDir) {
         QDir dir(dirName);
         QStringList pluginFileNames = getPluginFileNames(dir);
+
         foreach (QString pluginFile, pluginFileNames) {
             QString pluginAbsoluteFileName = dir.absoluteFilePath(pluginFile);
+
             if (!extensionFileNames.contains(pluginFile)) {
                 extensionFileNames.append(pluginFile);
                 extensionUniqueDir.insert(pluginFile, dir);
@@ -110,15 +122,20 @@ void ExtensionManager::autoload(ExtensionManager::ExtensionType type) {
     if (extensionAbsoluteFileNames.size() > 0) {
         // get the messages from Qt
         QStringList errorStrings;
+
         foreach (QString fileName, extensionAbsoluteFileNames) {
-            QPluginLoader pluginLoader(fileName);
+            QPluginLoader pluginLoader;
+            // from Qt doc: "causes all symbols in a library to be resolved when it is loaded, not simply when resolve() is called."
+            pluginLoader.setLoadHints(QLibrary::ResolveAllSymbolsHint);
+            pluginLoader.setFileName(fileName);
+
             QObject* plugin = pluginLoader.instance();
 
             if (!plugin) {
-                errorStrings << QString("Plugin " + fileName + ", error: " + pluginLoader.errorString());
+                errorStrings << QString("Plugin:\n\"" + pluginLoader.fileName() + "\"\n") << QString("Error:\n" + pluginLoader.errorString().remove(pluginLoader.fileName()).remove("(: ").remove(QRegularExpression("\\)$")) + "\n");
             }
             else {
-                errorStrings << QString("Plugin " + fileName + ", no error (you need to decrease your number of dependency level, or contact the CamiTK SDK developer team)");
+                errorStrings << QString("Plugin:\n\"" + pluginLoader.fileName() + "\"\n") << QString("No error (you need to increase your number of dependency level, or contact the CamiTK SDK developer team)");
             }
         }
 
@@ -132,8 +149,13 @@ void ExtensionManager::autoload(ExtensionManager::ExtensionType type) {
         osLibraryPathSeparator = ":";
 #endif
 
-        CAMITK_ERROR_ALT(tr("Extension Manager Opening Error: autoLoad plugin failed after %1 tries for the following extension(s):\n - %2\nList of library paths:\n - %3\nList of path:\n - %4\n").arg(
-                             QString::number(tryNr),                                                                                                                                                                        errorStrings.join("\n - "),                                                                                                                                                                                  Application::instance()->libraryPaths().join("\n - "),                                                                                                                                                                                                        QString(qgetenv(osLibraryPath.toStdString().c_str())).split(osLibraryPathSeparator.toStdString().c_str()).join("\n - ")))
+        CAMITK_ERROR_ALT(tr("Extension manager error:\nLoading %1 extension failed after %2 tries.\n\n%3\nList of library paths:\n - %4\n\nList of path:\n - %5")
+                         .arg((type == ACTION) ? "action" : ((type == COMPONENT) ? "component" : "viewer"),
+                              QString::number(tryNr),
+                              errorStrings.join("\n"),
+                              Application::instance()->libraryPaths().join("\n - "),
+                              QString(qgetenv(osLibraryPath.toStdString().c_str())).split(osLibraryPathSeparator.toStdString().c_str()).join("\n - "))
+                        )
 
     }
 
@@ -141,16 +163,24 @@ void ExtensionManager::autoload(ExtensionManager::ExtensionType type) {
     QSettings& settings = Application::getSettings();
     settings.beginGroup("UserExtensions");
     QStringList userRegisteredExtensions;
+
     switch (type) {
         case ACTION:
             userRegisteredExtensions = settings.value("actions", QVariant(QStringList())).toStringList();
             break;
+
         case COMPONENT:
             userRegisteredExtensions = settings.value("components", QVariant(QStringList())).toStringList();
             break;
+
+        case VIEWER:
+            userRegisteredExtensions = settings.value("viewers", QVariant(QStringList())).toStringList();
+            break;
+
         default:
             break;
     }
+
     settings.endGroup();
 
     foreach (QString userRegisteredExtensionFile, userRegisteredExtensions) {
@@ -161,11 +191,15 @@ void ExtensionManager::autoload(ExtensionManager::ExtensionType type) {
 
 // -------------------- loadExtension --------------------
 bool ExtensionManager::loadExtension(ExtensionManager::ExtensionType type, QString fileName) {
-    // First of all, check we have the private library directories in our session PATH
+    // First of all, add the private library directories in this session PATH
     initPrivateLibDirs();
 
     bool returnValue = false;
-    QPluginLoader pluginLoader(fileName);
+    QPluginLoader pluginLoader;
+    // from Qt doc: "causes all symbols in a library to be resolved when it is loaded, not simply when resolve() is called."
+    pluginLoader.setLoadHints(QLibrary::ResolveAllSymbolsHint);
+    pluginLoader.setFileName(fileName);
+    // load the plugin and get the instance
     QObject* extension = pluginLoader.instance();
 
     if (extension) {
@@ -188,6 +222,44 @@ bool ExtensionManager::loadExtension(ExtensionManager::ExtensionType type, QStri
             }
             break;
 
+            case VIEWER: {
+                ViewerExtension* ext = qobject_cast<ViewerExtension*> (extension);
+
+                if (ext) {
+                    ext->setLocation(fileName);
+                    ext->initResources();
+                    // initialize all viewers
+                    ext->init();
+
+                    // check that ext is managing a new class
+                    int i = 0;
+
+                    while (i < getViewerExtensionMap().size() && getViewerExtensionsList().at(i)->getViewerClassName() != ext->getViewerClassName()) {
+                        i++;
+                    }
+
+                    returnValue = (i >= getViewerExtensionMap().size());
+
+                    if (returnValue) {
+                        //-- register the filename
+                        getViewerExtensionMap().insert(fileName, ext);
+                        //-- register all viewers
+                        Application::registerAllViewers(ext);
+                    }
+                    else {
+                        CAMITK_INFO_ALT(tr("Extension Manager: duplicate extension management: viewer extension \"%1\" (in file \"%2\") declares management of \"%3\" while \"%4\" extension (loaded from file \"%5\") is already managing \"%6\".\nUsing extension in \"%7\" instead (higher priority) as only one viewer extension can manage an instance of viewer.")
+                                        .arg(ext->getName(),
+                                             fileName,
+                                             ext->getViewerClassName(),
+                                             getViewerExtensionsList().at(i)->getName(),
+                                             getViewerExtensionsList().at(i)->getLocation(),
+                                             ext->getViewerClassName(),
+                                             getViewerExtensionsList().at(i)->getLocation()))
+                    }
+                }
+            }
+            break;
+
             case COMPONENT: {
                 ComponentExtension* cp = qobject_cast<ComponentExtension*> (extension);
 
@@ -207,6 +279,7 @@ bool ExtensionManager::loadExtension(ExtensionManager::ExtensionType type, QStri
                         // (cannot do that in the constructor because the virtual symbol table seems to be confused!)
                         foreach (QString ext, cp->getFileExtensions()) {
                             ComponentExtension* existingComponentExtension = getComponentExtensionMap().value(ext);
+
                             if (existingComponentExtension != nullptr) {
                                 CAMITK_INFO_ALT(tr("Extension Manager: duplicate extension management: component extension \"%1\" (in file \"%2\") declares management of \"%3\" file extension while \"%4\" extension (loaded from file \"%5\") is already managing \"%6\".\nUsing extension in \"%7\" instead (higher priority) as only one component extension can manage a give file extension.")
                                                 .arg(cp->getName(),
@@ -244,10 +317,18 @@ bool ExtensionManager::loadExtension(ExtensionManager::ExtensionType type, QStri
 
         // try to load the missing shared object directly from private dir (give absolute path)
         QRegExp libname("\\((lib.*): cannot open shared object file");
+
         if (libname.indexIn(pluginLoader.errorString(), 0) != -1) {
-            QString privateLibToLoad = Core::getGlobalInstallDir() + "/lib/" + QString(Core::shortVersion) + "/" + libname.cap(1);
+            QString privateLibToLoad = Core::getGlobalInstallDir() + "/" + QString(Core::libDir) + "/" + QString(Core::shortVersion) + "/" + libname.cap(1);
             QLibrary privateLib(privateLibToLoad);
-            privateLib.load();
+            bool foundInLibDir = privateLib.load();
+
+            if (!foundInLibDir && QString(Core::libDir) != "lib") {
+                // now try "lib/" directly
+                privateLibToLoad = Core::getGlobalInstallDir() + "/" + QString(Core::libDir) + "/" + QString(Core::shortVersion) + "/" + libname.cap(1);
+                privateLib.setFileName(privateLibToLoad);
+                privateLib.load();
+            }
         }
     }
 
@@ -260,24 +341,56 @@ QString ExtensionManager::getInstallationString(QString file, const QString& glo
     QString whichInstallDir;
     QDir dir(QFileInfo(file).absolutePath());
 
-    // go up three levels (one for the extension name, one for the camitk short version name, one for the lib name)
+    // go up three levels
+    // - go up one level for the extension name (e.g. "actions")
     dir.cdUp();
+    // - go up one level for the camitk short version name ("camitk-x.x")
     dir.cdUp();
+    // - go up one level for the lib dirname ("lib")
     dir.cdUp();
 
     QString absolutePath = dir.absolutePath();
+
     if (absolutePath == currentWorkingDir) {
         whichInstallDir = "[W]";
     }
-    else if (absolutePath == userInstallDir) {
-        whichInstallDir = "[L]";
-    }
-    else if (absolutePath == globalInstallDir) {
-        whichInstallDir = "[G]";
-    }
     else {
-        whichInstallDir = "[U]";
+        if (absolutePath == userInstallDir) {
+            whichInstallDir = "[L]";
+        }
+        else {
+            if (absolutePath == globalInstallDir) {
+                whichInstallDir = "[G]";
+            }
+            else {
+                // Take special case of existance of a subdirectory in Core::libDir
+                // add one cdUp() for each "/" inside Core::libDir (to take into account any multiarch configuration that
+                //   landed in CMAKE_INSTALL_LIBDIR, eg. "lib/x86_64-linux-gnu")
+                for (int i = 0; i < QString(Core::libDir).count(QLatin1Char('/')); i++) {
+                    dir.cdUp();
+                }
+                absolutePath = dir.absolutePath();
+
+                if (absolutePath == currentWorkingDir) {
+                    whichInstallDir = "[W]";
+                }
+                else {
+                    if (absolutePath == userInstallDir) {
+                        whichInstallDir = "[L]";
+                    }
+                    else {
+                        if (absolutePath == globalInstallDir) {
+                            whichInstallDir = "[G]";
+                        }
+                        else {
+                            whichInstallDir = "[U]";
+                        }
+                    }
+                }
+            }
+        }
     }
+
     return whichInstallDir;
 }
 
@@ -327,7 +440,7 @@ ComponentExtension* ExtensionManager::getComponentExtension(QString extOrName) {
 // -------------------- getComponentExtensionsList --------------------
 const QList< ComponentExtension* > ExtensionManager::getComponentExtensionsList() {
     QList< ComponentExtension* > sortedList = getComponentExtensionMap().values().toSet().toList(); // to remove duplicates
-    qSort(sortedList.begin(), sortedList.end(), ExtensionManager::componentExtensionLessThan);
+    std::sort(sortedList.begin(), sortedList.end(), ExtensionManager::componentExtensionLessThan);
 
     return sortedList;
 }
@@ -335,7 +448,7 @@ const QList< ComponentExtension* > ExtensionManager::getComponentExtensionsList(
 // -------------------- getDataDirectoryComponentsList --------------------
 const QList< ComponentExtension* > ExtensionManager::getDataDirectoryComponentsList() {
     QList< ComponentExtension* > sortedList = getDataDirectoryComponentExtensionMap().values().toSet().toList(); // to remove duplicates
-    qSort(sortedList.begin(), sortedList.end(), ExtensionManager::componentExtensionLessThan);
+    std::sort(sortedList.begin(), sortedList.end(), ExtensionManager::componentExtensionLessThan);
 
     return sortedList;
 }
@@ -361,7 +474,7 @@ QMap<QString, ActionExtension*>&    ExtensionManager::getActionExtensionMap() {
 // -------------------- getActionExtensionsList --------------------
 const QList< ActionExtension* > ExtensionManager::getActionExtensionsList() {
     QList< ActionExtension* > sortedList = getActionExtensionMap().values().toSet().toList(); // to remove duplicates
-    qSort(sortedList.begin(), sortedList.end(), ExtensionManager::actionExtensionLessThan);
+    std::sort(sortedList.begin(), sortedList.end(), ExtensionManager::actionExtensionLessThan);
 
     return sortedList;
 }
@@ -380,13 +493,61 @@ void ExtensionManager::unloadAllActionExtensions() {
     }
 }
 
+// -------------------- getViewerExtensionMap --------------------
+QMap<QString, ViewerExtension*>& camitk::ExtensionManager::getViewerExtensionMap() {
+    static QMap<QString, ViewerExtension*> viewerExtensionMap;
+
+    return viewerExtensionMap;
+}
+
+// -------------------- unloadAllViewerExtensions --------------------
+void camitk::ExtensionManager::unloadAllViewerExtensions() {
+    QList<QString> allExtensions = getViewerExtensionMap().keys();
+
+    while (!allExtensions.isEmpty()) {
+        unloadViewerExtension(allExtensions.takeFirst());
+    }
+}
+
+// -------------------- getViewerExtensionsList --------------------
+const QList<ViewerExtension*> camitk::ExtensionManager::getViewerExtensionsList() {
+    QList< ViewerExtension* > sortedList = getViewerExtensionMap().values().toSet().toList(); // to remove duplicates
+    std::sort(sortedList.begin(), sortedList.end(), ExtensionManager::viewerExtensionLessThan);
+
+    return sortedList;
+}
+
+// -------------------- getViewerExtension --------------------
+ViewerExtension* camitk::ExtensionManager::getViewerExtension(QString file) {
+    return getViewerExtensionMap().value(file);
+}
+
+// -------------------- unloadViewerExtension --------------------
+bool ExtensionManager::unloadViewerExtension(QString fileName) {
+    if (getViewerExtensionMap().contains(fileName)) {
+        ViewerExtension* ext = getViewerExtensionMap().value(fileName);
+        // unregister extension
+        getViewerExtensionMap().remove(fileName);
+        // unregister actions from application
+        Application::unregisterAllViewers(ext);
+        // delete extensions (and all its actions)
+        delete ext;
+
+        return true;
+    }
+    else {
+        return false;
+    }
+}
+
 // -------------------- registerFileExtension --------------------
 void ExtensionManager::registerFileExtension(QString fileExtension) {
+
+#ifdef WIN32
     // we get the application name and its binary path for the association
     QString appName = QApplication::applicationName(); // used for windows registry
     QString appFilePath = QApplication::applicationFilePath(); // needed for associating file for opening command
 
-#ifdef WIN32
     // WINDOWS ONLY
     appFilePath.replace("/", "\\");
     // Associate the file extension with the current application for opening
@@ -416,12 +577,14 @@ QStringList ExtensionManager::getExtensionFilter() {
     QStringList pluginFilter;
     // linux: .so, windows: .dll, macOs: .dylib
     pluginFilter << "*.so." + QString(Core::soVersion) << "*." + QString(Core::soVersion) + ".dylib";
+
     if (Core::isDebugBuild()) {
         pluginFilter << "*" + QString(Core::debugPostfix) + ".dll";
     }
     else {
         pluginFilter << "*.dll";
     }
+
     return pluginFilter;
 }
 
@@ -443,8 +606,10 @@ QStringList ExtensionManager::getPluginFileNames(QDir extensionsDir) {
         foreach (QString debugDLL, pluginFileNamesDebugMSVC) {
             pluginFileNames.removeAll(debugDLL);
         }
+
         return pluginFileNames;
     }
+
 #endif
 
     return pluginFileNames;
@@ -455,10 +620,7 @@ QStringList ExtensionManager::getPluginFileNames(QDir extensionsDir) {
 bool ExtensionManager::unloadActionExtension(QString fileName) {
     if (getActionExtensionMap().contains(fileName)) {
         ActionExtension* ext = getActionExtensionMap().value(fileName);
-        //-- unregister all actions
-        foreach (Action* action, ext->getActions()) {
-            getActionExtensionMap().remove(action->getName());
-        }
+
         //-- unregister extension
         getActionExtensionMap().remove(fileName);
         // -- unregister actions from application
@@ -525,6 +687,33 @@ bool ExtensionManager::unloadComponentExtension(QString extOrName) {
     return unloaded;
 }
 
+// -------------------- addLibraryPath --------------------
+void ExtensionManager::addLibraryPath(QDir potentialPath, QString potentialLibDir) {
+
+    if (potentialPath.cd(potentialLibDir + "/" + QString(Core::shortVersion))) {
+        QByteArray confirmedLibDirPath = potentialPath.canonicalPath().toUtf8();
+
+#ifdef WIN32 // for Windows
+        QByteArray path = qgetenv("PATH");
+        path.append(";");
+        path.append(confirmedLibDirPath);
+        path.append(";");
+        path.append(confirmedLibDirPath + "/actions");
+        path.append(";");
+        path.append(confirmedLibDirPath + "/components");
+        path.append(";");
+        path.append(confirmedLibDirPath + "/viewers");
+        qputenv("PATH", path);
+#endif
+        // update the application / qt plugins library path
+        Application::instance()->addLibraryPath(confirmedLibDirPath);
+        Application::instance()->addLibraryPath(confirmedLibDirPath + "/actions");
+        Application::instance()->addLibraryPath(confirmedLibDirPath + "/components");
+        Application::instance()->addLibraryPath(confirmedLibDirPath + "/viewers");
+    }
+}
+
+
 // -------------------- initPrivateLibDirs --------------------
 void ExtensionManager::initPrivateLibDirs() {
     // Add the private lib dirs to the current application environment's PATH variable
@@ -533,65 +722,22 @@ void ExtensionManager::initPrivateLibDirs() {
     static bool alreadyInitialized = false;
 
     if (!alreadyInitialized) {
-        // Build directory
-        QDir privateBuildLibDir(Core::getCurrentWorkingDir());
-        if (privateBuildLibDir.cd("lib/" + QString(Core::shortVersion))) {
-            QByteArray privateBuildLibDirPath = privateBuildLibDir.canonicalPath().toUtf8();
-#ifdef WIN32 // for Windows
-            QByteArray path = qgetenv("PATH");
-            path.append(";");
-            path.append(privateBuildLibDirPath);
-            path.append(";");
-            path.append(privateBuildLibDirPath + "/actions");
-            path.append(";");
-            path.append(privateBuildLibDirPath + "/components");
-            qputenv("PATH", path);
-#endif
-            // update the application / qt plugins library path
-            Application::instance()->addLibraryPath(privateBuildLibDirPath);
-            Application::instance()->addLibraryPath(privateBuildLibDirPath + "/actions");
-            Application::instance()->addLibraryPath(privateBuildLibDirPath + "/components");
-        }
+        // Build directory / current working directory
+        addLibraryPath(Core::getCurrentWorkingDir(), QString(Core::libDir));
 
         // Local install directory
-        QDir privateLocalLibDir(Core::getUserInstallDir());
-        if (privateLocalLibDir.cd("lib/" + QString(Core::shortVersion))) {
-            QByteArray privateLocalLibDirPath = privateLocalLibDir.canonicalPath().toUtf8();
-#ifdef WIN32 // for Windows
-            QByteArray path = qgetenv("PATH");
-            path.append(";");
-            path.append(privateLocalLibDirPath);
-            path.append(";");
-            path.append(privateLocalLibDirPath + "/actions");
-            path.append(";");
-            path.append(privateLocalLibDirPath + "/components");
-            qputenv("PATH", path);
-#endif
-            // update the application / qt plugins library path
-            Application::instance()->addLibraryPath(privateLocalLibDirPath);
-            Application::instance()->addLibraryPath(privateLocalLibDirPath + "/actions");
-            Application::instance()->addLibraryPath(privateLocalLibDirPath + "/components");
-        }
+        addLibraryPath(Core::getUserInstallDir(), QString(Core::libDir));
 
         // Global install directory
-        QDir privateGlobalLibDir(Core::getGlobalInstallDir());
-        if (privateGlobalLibDir.cd("lib/" + QString(Core::shortVersion))) {
-            QByteArray privateGlobalLibDirPath = privateGlobalLibDir.canonicalPath().toUtf8();
-#ifdef WIN32 // for Windows
-            QByteArray path = qgetenv("PATH");
-            path.append(";");
-            path.append(privateGlobalLibDirPath);
-            path.append(";");
-            path.append(privateGlobalLibDirPath + "/actions");
-            path.append(";");
-            path.append(privateGlobalLibDirPath + "/components");
-            qputenv("PATH", path);
-#endif
-            // update the application / qt plugins library path
-            Application::instance()->addLibraryPath(privateGlobalLibDirPath);
-            Application::instance()->addLibraryPath(privateGlobalLibDirPath + "/actions");
-            Application::instance()->addLibraryPath(privateGlobalLibDirPath + "/components");
+        addLibraryPath(Core::getGlobalInstallDir(), QString(Core::libDir));
+
+        // take multiarch into account
+        if (QString(Core::libDir) != "lib") {
+            addLibraryPath(Core::getCurrentWorkingDir(), "lib");
+            addLibraryPath(Core::getUserInstallDir(), "lib");
+            addLibraryPath(Core::getGlobalInstallDir(), "lib");
         }
+
         alreadyInitialized = true;
     }
 }
@@ -606,6 +752,11 @@ bool ExtensionManager::actionExtensionLessThan(ActionExtension* left, ActionExte
     return left->getName() < right->getName();
 }
 
+// -------------------- viewerExtensionLessThan --------------------
+bool ExtensionManager::viewerExtensionLessThan(ViewerExtension* left, ViewerExtension* right) {
+    return left->getName() < right->getName();
+}
+
 }
 
 
