@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -28,8 +28,8 @@
 
 // -- Core stuff
 #include "CamiTKAPI.h"
-#include <HistoryItem.h>
 #include <Application.h>
+#include <Component.h>
 
 #include <QSet>
 #include <QWidget>
@@ -41,6 +41,7 @@ namespace camitk {
 
 class ActionExtension;
 class Property;
+class HistoryItem;
 
 /**
  * @ingroup group_sdk_libraries_core_action
@@ -119,7 +120,7 @@ class Property;
  *
  * You can add any number of tags using the method addTag().
  *
- * If ActionWidget is not what your need, a typical getWidget() method should use the lazy instanciation pattern to instanciate
+ * If ActionWidget is not what your need, a typical getWidget() method should use the lazy instantiation pattern to instantiate
  * MyVerySpecialActionWidget the first time it is called, and call the MyVerySpecialActionWidget instance's updateTargets() method
  * for any subsequent calls. Something like:
  * \code
@@ -136,7 +137,7 @@ class Property;
  * \endcode
  * But of course you can also use any kind of widget you like. ActionWidget is just defining
  * a default widget for an action.
- * If your action does not have any GUI/parameters, just the getWidget() method in order to return NULL.
+ * If your action does not have any GUI/parameters, just the getWidget() method in order to return nullptr.
  *
  * By default the properties/parameters are not automatically updated when the user change the default widget,
  * they are updated only when the user click on the apply button of the default widget.
@@ -241,7 +242,7 @@ public slots:
      * The parent widget is used if the action is embedded, see class description for more information about the algorithm.
      * This method cannot be redefined in inherited class.
      */
-    ApplyStatus trigger(QWidget* parent = nullptr);
+    camitk::Action::ApplyStatus trigger(QWidget* parent = nullptr);
 
     /**
      * This method is called when the action has to be applied on the target list (get the target lists using getTargets())
@@ -252,7 +253,7 @@ public slots:
      * method
      * @return The status of the apply method.
      */
-    virtual ApplyStatus apply() = 0;
+    virtual camitk::Action::ApplyStatus apply() = 0;
 
     /**
      * This method is called whenever the action has to be applied on the target list (like the apply()) method
@@ -261,7 +262,7 @@ public slots:
      * The application's history of actions allows one to export the saved actions as an XML file for scripting or replaying it.
      * @return The status of the apply method.
      **/
-    ApplyStatus applyAndRegister();
+    camitk::Action::ApplyStatus applyAndRegister();
 
 public:
 
@@ -304,8 +305,11 @@ public:
      *
      *  To add a shortcut, simply call getQAction()->setShortcut(..) in the action constructor.
      *  To make this shortcut available for any windows of the application, call getQAction()->setShortcutContext(Qt::ApplicationShortcut);
+     *
+     *  Override this method if your action can be toggled on/off (for instance an action that modifies the visibility
+     *  of a component). This method is called in the Component class to build the component's action menu.
      */
-    QAction* getQAction();
+    virtual QAction* getQAction(Component* target = nullptr);
 
     /// get the name of the action
     QString getName() const {
@@ -344,7 +348,7 @@ public:
     /// @name Method specific to an action.
     /// @{
     /** This method has to be redefined in your Action only if:
-      * - you do not have any widget to control your action (i.e. getWidget() will have to return NULL),
+      * - you do not have any widget to control your action (i.e. getWidget() will have to return nullptr),
       * - you do not use the default ActionWidget but another one.
       *
       * In the second case, it is strongly recommanded to have a code similar to this:
@@ -372,6 +376,9 @@ public:
 
     /// the currently selected and valid (regarding the component property) components, for which this action is called
     const ComponentList getTargets() const;
+
+    /// update the target list using the currently selected components
+    void updateTargets();
     ///@}
 
     /// @name Property management
@@ -384,11 +391,11 @@ public:
 
     /** Get a Property given its name
      *  @param name the property name
-     *  @return NULL if the name does not match any property name
+     *  @return nullptr if the name does not match any property name
      *
      *  @see Property
      */
-    Q_INVOKABLE virtual Property* getProperty(QString name);
+    Q_INVOKABLE virtual camitk::Property* getProperty(QString name);
 
     /** Add a new parameter to the action, using the CamiTK property class.
      * If the parameter already exist, it will just change its value.
@@ -399,7 +406,7 @@ public:
      * @return false if the Qt Meta Object property was added by this method (otherwise the property was already defined and true is returned if it was successfully updated)
      */
     virtual bool addParameter(Property*);
-    ///}@
+    /// @}
 
     /// @name Frame management
     /// @{
@@ -417,7 +424,7 @@ public:
      * @param policy The frame policy to use on the target.
      */
     void applyTargetPosition(Component* input, Component* target, Application::TargetPositionningPolicy policy);
-    /// }@
+    /// @}
 
 protected:
     /** @name Generic action attributes setters
@@ -572,7 +579,7 @@ private:
 }
 
 // -------------------- declare the interface for QPluginLoader --------------------
-Q_DECLARE_INTERFACE(camitk::Action, "TIMC-IMAG.Action/2.1") //TODO use svn version?
+Q_DECLARE_INTERFACE(camitk::Action, "TIMC-IMAG.Action/2.1")
 
 #endif // ACTION_H
 
