@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -29,6 +29,7 @@
 #include "MainWindow.h"
 #include "ExtensionManager.h"
 #include "Action.h"
+#include "Viewer.h"
 #include "HistoryComponent.h"
 #include "ImageComponent.h"
 #include "Property.h"
@@ -66,6 +67,7 @@ int Application::argc = 0;
 char** Application::argv = nullptr;
 QTranslator* Application::translator = nullptr;
 PropertyObject* Application::propertyObject = nullptr;
+Action* Application::currentAction = nullptr;
 
 // ----------------- constructor --------------------
 Application::Application(QString name, int& theArgc, char** theArgv, bool autoloadExtensions, bool registerFileExtension) : QApplication(theArgc, theArgv) {
@@ -110,7 +112,7 @@ Application::Application(QString name, int& theArgc, char** theArgv, bool autolo
 
     // the last used directory
     QDir defaultDir(Core::getTestDataDir());
-    lastUsedDirectory = settings.value("lastUsedDirectory", defaultDir.absolutePath()).toString();
+    lastUsedDirectory.setPath(settings.value("lastUsedDirectory", defaultDir.absolutePath()).toString());
     settings.endGroup();
 
     //-- register file association with this application for opening
@@ -181,7 +183,7 @@ Application::Application(QString name, int& theArgc, char** theArgv, bool autolo
     propertyObject->installEventFilter(this);
 
     // trigger change for all the values
-    eventFilter(this, new QEvent(QEvent::DynamicPropertyChange));
+    Application::eventFilter(this, new QEvent(QEvent::DynamicPropertyChange));
 }
 
 // ----------------- destructor --------------------
@@ -204,14 +206,16 @@ QString Application::getName() {
 
 // ----------------- quitting --------------------
 void Application::quitting() {
-    // this is connect to the aboutToQuit signal from QApplication
+    // this is connected to the aboutToQuit signal from QApplication
     // it should contain all the code that frees the resources
 
     // delete all actions (they are instantiated when the extension is loaded)
     ExtensionManager::unloadAllActionExtensions();
+    ExtensionManager::unloadAllViewerExtensions();
 
-    if (translator) {
-        delete translator;    //delete instance of internationalization support
+    if (translator != nullptr) {
+        // delete instance of internationalization support
+        delete translator;
     }
 }
 
@@ -219,6 +223,7 @@ void Application::quitting() {
 bool Application::notify(QObject* receiver, QEvent* event) {
     bool done = true;
     std::exception_ptr otherException;
+
     try {
         done = QApplication::notify(receiver, event);
     }
@@ -228,6 +233,7 @@ bool Application::notify(QObject* receiver, QEvent* event) {
     catch (...) {
         CAMITK_ERROR(tr("Caught an unknown exception"))
         otherException = std::current_exception();
+
         try {
             if (otherException) {
                 std::rethrow_exception(otherException);
@@ -289,8 +295,11 @@ void Application::setMainWindow(MainWindow* mw) {
 
 // ----------------- getMainWindow --------------------
 MainWindow* Application::getMainWindow() {
-    if (!mainWindow) {
+    if (mainWindow == nullptr && dynamic_cast<Application*>(qApp) != nullptr) {
+        // this is the first time getMainWindow() is called and no MainWindow were created
+        // In this case, force the creation of a default MainWindow by sending nullptr
         dynamic_cast<Application*>(qApp)->setMainWindow(nullptr);
+        // and then returns it
     }
 
     return mainWindow;
@@ -303,19 +312,33 @@ QSettings& Application::getSettings() {
 
 // ----------------- exec --------------------
 int Application::exec() {
-    if (!mainWindow) {
+    if (mainWindow == nullptr && dynamic_cast<Application*>(qApp) != nullptr) {
         dynamic_cast<Application*>(qApp)->setMainWindow(nullptr);
     }
 
-    mainWindow->aboutToShow();
-    mainWindow->show();
+    // if mainWindow was nullptr when entering this method, it should have
+    // been created by the call to setMainWindow(nullptr) above
+    if (mainWindow != nullptr) {
+        mainWindow->aboutToShow();
+        mainWindow->show();
+    }
+    else {
+        CAMITK_ERROR_ALT(tr("Cannot create MainWindow instance"));
+    }
 
     return QApplication::exec();
 }
 
 // ----------------- refresh --------------------
 void Application::refresh() {
-    mainWindow->refresh();
+    if (mainWindow != nullptr) {
+        mainWindow->refresh();
+    }
+
+    // now that every viewer was refresh, clear the interface node modification flag
+    for (Component* c : Application::getAllComponents()) {
+        c->setNodeModified(false);
+    }
 }
 
 // ----------------- showStatusBarMessage --------------------
@@ -478,7 +501,7 @@ Component* Application::open(const QString& fileName) {
                 addRecentDocument(fileName);
                 showStatusBarMessage(tr(QString("File " + fileName + " successfully loaded").toStdString().c_str()));
 
-                CAMITK_WARNING_IF_ALT((!comp->isTopLevel()), tr("Instanciating a NON top level component."))
+                CAMITK_WARNING_IF_ALT((!comp->isTopLevel()), tr("Instantiating a NON top level component."))
 
                 // refresh all viewers
                 refresh();
@@ -509,6 +532,7 @@ Component* Application::open(const QString& fileName) {
 
             // try harder
             otherException = std::current_exception();
+
             try {
                 if (otherException) {
                     std::rethrow_exception(otherException);
@@ -538,6 +562,7 @@ Component* Application::openDirectory(const QString& dirName, const QString& plu
 
     if (cp != nullptr) {
         std::exception_ptr otherException;
+
         // Ask the plugin instance to create the Component instance
         try {
             comp = cp->open(QDir(dirName).absolutePath());
@@ -570,6 +595,7 @@ Component* Application::openDirectory(const QString& dirName, const QString& plu
 
             // try harder
             otherException = std::current_exception();
+
             try {
                 if (otherException) {
                     std::rethrow_exception(otherException);
@@ -660,17 +686,22 @@ bool Application::save(Component* component) {
 
     // -- check the validity of the plugin
     if (!componentExtension) {
-        CAMITK_ERROR_ALT(tr("Saving Error: cannot find the appropriate component plugin for saving component:\n"
-                            "\"%1\"\n"
-                            "In file:\n"
-                            "\"%1\" (extension \"%2\" or \"%3\")\n"
+        QString extensionFileSuffix = "\"" + fileInfo.suffix() + "\"";
+        if (fileInfo.completeSuffix() != fileInfo.suffix()) {
+            extensionFileSuffix += "(or \"" + fileInfo.completeSuffix() + "\")";
+        }
+        CAMITK_ERROR_ALT(tr("Saving Error: cannot find the appropriate component extension for saving component: \"%1\"\n"
+                            "Of type: \"%2\"\n"
+                            "In file: \"%3\"\n"
+                            "Using suffix: %4\n"
                             "To solve this problem, make sure that:\n"
-                            " - A corresponding valid plugin is present in one of the component directories: \"%4\"\n"
+                            " - A component extension that manages %4 suffix is present in one of the component extension directories: [%5]\n"
                             " - And either your application is initialized with the autoloadExtensions option\n"
-                            " - Or your correctly registered your component in the CamiTK settings")
+                            "   or you correctly registered your component in the CamiTK settings")
                          .arg(component->getName(),
-                              fileInfo.suffix(),
-                              fileInfo.completeSuffix(),
+                              component->metaObject()->className(),
+                              fileInfo.filePath(),
+                              extensionFileSuffix,
                               Core::getComponentDirectories().join(", ")))
         return false;
     }
@@ -749,15 +780,31 @@ bool actionLessThan(const camitk::Action* a1, const camitk::Action* a2) {
     return a1->getName() < a2->getName();
 }
 
+// ---------------- viewerLessThan ----------------
+bool viewerLessThan(const camitk::Viewer* v1, const camitk::Viewer* v2) {
+    // This method is needed by qsort in the sort method to sort viewer by name
+    return v1->getName() < v2->getName();
+}
+
+
 // ---------------- sort ----------------
 ActionList Application::sort(ActionSet actionSet) {
     // sort actions by name
-    ActionList actionList = actionSet.toList();
-    qSort(actionList.begin(), actionList.end(), actionLessThan);
+    ActionList actionList = actionSet.values();
+    std::sort(actionList.begin(), actionList.end(), actionLessThan);
 
     return actionList;
 }
 
+ViewerList Application::sort(ViewerSet viewerSet) {
+    // sort actions by name
+    ViewerList viewerList = viewerSet.values();
+    std::sort(viewerList.begin(), viewerList.end(), viewerLessThan);
+
+    return viewerList;
+}
+
+
 // ---------------- getAction ----------------
 Action* Application::getAction(QString name) {
     return getActionMap().value(name);
@@ -789,7 +836,7 @@ ActionList Application::getActions(Component* component) {
 
 // ---------------- getActions ----------------
 ActionList Application::getActions(ComponentList cList) {
-    // if this is an empty list, return all action not based on a component
+    // if this is an empty list, return all actions not using any component
     if (cList.size() == 0) {
         return getActions(nullptr);
     }
@@ -819,11 +866,158 @@ ActionList Application::getActions(ComponentList selComp, QString tag) {
     }
 
     // sort and return
-    qSort(actions.begin(), actions.end(), actionLessThan);
+    std::sort(actions.begin(), actions.end(), actionLessThan);
     return actions;
 
 }
 
+// -------------------- getViewerMap --------------------
+QMap<QString, Viewer*>& Application::getViewerMap() {
+    static QMap<QString, Viewer*> viewerMap;
+
+    return viewerMap;
+}
+
+// -------------------- getViewer --------------------
+Viewer* Application::getViewer(QString name) {
+    return getViewerMap().value(name);
+}
+
+// -------------------- registerViewer --------------------
+bool Application::registerViewer(Viewer* viewer) {
+    // first check if a viewer with the same name is not already registered
+    if (getViewerMap().contains(viewer->getName())) {
+        // if it is already registered, find out more information about the viewer extension
+        // that manages the already registered viewer
+        QString extensionName = "";
+        ViewerExtension* extension = getViewerExtension(viewer);
+
+        if (extension != nullptr) {
+            extensionName = extension->getName();
+            CAMITK_ERROR_ALT(tr("Cannot register viewer: %1 (description: \"%2\")\n"
+                                "viewer of same name already registered in extension %3")
+                             .arg(viewer->getName(), viewer->getDescription(), extensionName))
+        }
+        else {
+            // should never happen
+            CAMITK_ERROR_ALT(tr("Cannot register viewer: %1 (description: \"%2\")\n"
+                                "viewer of same name already registered in an unknown extension...")
+                             .arg(viewer->getName(), viewer->getDescription()))
+        }
+
+        return false;
+    }
+    else {
+        // everything is alright register the viewer
+        getViewerMap().insert(viewer->getName(), viewer);
+        return true;
+    }
+}
+
+// -------------------- registerAllViewers --------------------
+int Application::registerAllViewers(ViewerExtension* ext) {
+    int registered = 0;
+
+    // register all the viewer instances in this extension
+    for (Viewer* viewer : ext->getViewers()) {
+        if (registerViewer(viewer)) {
+            registered++;
+        }
+    }
+
+    return registered;
+}
+
+// -------------------- getViewers --------------------
+ViewerList Application::getViewers(Component* component) {
+    ViewerSet viewers;
+
+    if (component) {
+        QStringList componentHierarchy = component->getHierarchy();
+
+        foreach (Viewer* currentViewer, Application::getViewers()) {
+            foreach (QString compType, currentViewer->getComponents()) {
+                if (componentHierarchy.contains(compType)) {
+                    viewers.insert(currentViewer);
+                }
+            }
+        }
+    }
+    else {
+        foreach (Viewer* currentViewer, Application::getViewers()) {
+            foreach (QString compType, currentViewer->getComponents()) {
+                if (compType.isEmpty()) {
+                    viewers.insert(currentViewer);
+                }
+            }
+        }
+    }
+
+    return sort(viewers);
+}
+
+// -------------------- getNewViewer --------------------
+Viewer* Application::getNewViewer(QString name, QString className) {
+
+    int i = 0;
+    QList<ViewerExtension*> vl = ExtensionManager::getViewerExtensionsList();
+
+    while (i < vl.size() && vl.at(i)->getViewerClassName() != className) {
+        i++;
+    }
+
+    if (i < vl.size()) {
+        ViewerExtension* ve = vl.at(i);
+        return ve->getNewInstance(name);
+    }
+    else {
+        CAMITK_ERROR_ALT(tr("Cannot find a viewer extension that manage viewers of type \"%1\"\n").arg(className))
+        return nullptr;
+    }
+}
+
+// -------------------- getViewers --------------------
+const ViewerList Application::getViewers() {
+    return getViewerMap().values();
+}
+
+// -------------------- unregisterAllViewers --------------------
+int Application::unregisterAllViewers(ViewerExtension* ext) {
+    int unregistered = 0;
+
+    foreach (Viewer* viewer, ext->getViewers()) {
+        getViewerMap().remove(viewer->getName());
+        unregistered++;
+    }
+
+    return unregistered;
+}
+
+// -------------------- getViewerExtension --------------------
+ViewerExtension* Application::getViewerExtension(Viewer* viewer) {
+
+    int i = 0;
+    bool found = false;
+    ViewerExtension* extension = nullptr;
+
+    while (i < ExtensionManager::getViewerExtensionsList().size() && !found) {
+        int j = 0;
+
+        while (j < ExtensionManager::getViewerExtensionsList().at(i)->getViewers().size() && !found) {
+            found = (ExtensionManager::getViewerExtensionsList().at(i)->getViewers().at(j)->getName() == viewer->getName());
+            j++;
+        }
+
+        i++;
+    }
+
+    if (found) {
+        extension = ExtensionManager::getViewerExtensionsList().at(i - 1);
+    }
+
+    return extension;
+}
+
 // ---------- isAlive ----------
 bool Application::isAlive(Component* comp) {
     return getAllComponents().contains(comp);
@@ -917,6 +1111,16 @@ void Application::clearSelectedComponents() {
     getSelectedComponentList().clear();
 }
 
+// -------------------- setSelectedAction --------------------
+void Application::setTriggeredAction(Action* action) {
+    currentAction = action;
+}
+
+// -------------------- getSelectedAction --------------------
+Action* Application::getTriggeredAction() {
+    return currentAction;
+}
+
 // -------------------- getHistory --------------------
 QStack<HistoryItem>& Application::getHistory() {
     // static singleton declaration
@@ -1014,6 +1218,7 @@ void Application::saveHistoryAsSXML() {
         // action parameters
         QDomElement parametersElement = doc.createElement("camitk:parameters");
         actionElement.appendChild(parametersElement);
+
         if (!action->dynamicPropertyNames().isEmpty()) {
 
             foreach (QByteArray actionParameter, action->dynamicPropertyNames()) {
@@ -1234,9 +1439,9 @@ QString Application::getSelectedLanguage() {
     QSettings& settings(Application::getSettings());
     settings.beginGroup(Application::getName() + ".Application");
     // Get the language an d flag for the current Application instance
-    QString language = settings.value("language").toString();
+    //QString language = settings.value("language").toString();
     QString languageAbreviation = settings.value("languageAbreviation").toString();
-    QString flagFile = settings.value("flagFilename").toString();
+    //QString flagFile = settings.value("flagFilename").toString();
     settings.endGroup();
 
     return languageAbreviation;
@@ -1265,7 +1470,7 @@ void Application::initResources() {
 
 // -------------------- createProperties --------------------
 void Application::createProperties() {
-    // instanciate the application property object
+    // instantiate the application property object
     propertyObject = new PropertyObject(getName());
 
     Property* property;
@@ -1288,9 +1493,10 @@ void Application::createProperties() {
     property->setEnumTypeName("LogLevel");
     // Set a custom list of GUI names
     QStringList enumGuiText;
+
     for (const InterfaceLogger::LogLevel l : {
-    InterfaceLogger::NONE, InterfaceLogger::ERROR, InterfaceLogger::WARNING, InterfaceLogger::INFO, InterfaceLogger::TRACE
-}) {
+                InterfaceLogger::NONE, InterfaceLogger::ERROR, InterfaceLogger::WARNING, InterfaceLogger::INFO, InterfaceLogger::TRACE
+            }) {
         enumGuiText << Log::getLevelAsString(l);
     }
     property->setAttribute("enumNames", enumGuiText);
