@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -43,7 +43,9 @@ class vtkObject;
 namespace camitk {
 class MainWindow;
 class ActionExtension;
+class ViewerExtension;
 class Action;
+class Viewer;
 class HistoryItem;
 class ComponentExtension;
 class Property;
@@ -58,21 +60,38 @@ class PropertyObject;
  *  The constructor can take the command line arguments.
  *  It can also be asked not to load the extensions automatically,see Application().
  *
- *  If you do not have a specific MainWindow extension, then the default CamiTK MainWindow is used, see setMainWindow()
+ *  If you do not have a specific MainWindow extension, then the default CamiTK MainWindow is used, see setMainWindow().
+ *
+ * This class manages all application-level instances, structures and all. This explains the number of _static_ methods
+ * in this class.
+ *
+ * It manages:
+ * - the registered/loaded action extensions and all the actions
+ * - the registered/loaded component extensions and all the component instances
+ * - the registered/loaded viewer extensions and all the viewer instances
+ * - the refresh mechanism
+ * - the current selection (selected components)
+ * - the recently opened documents
+ * - the application language/internationalization settings
+ * - the opening/closing/saving of components
+ * - the main window
+ * - the history of applied actions (including saving it as a CamitK SCXML document)
+ * - some application level settings
+ *
  */
 class CAMITK_API Application : public QApplication {
     Q_OBJECT
 
 public:
 
-    /// @enum TargetPositionningPolicy: Policy to determine how a newly instanciated component's frame should be initialized regarding of its parent's component.
+    /// @enum TargetPositionningPolicy: Policy to determine how a newly instantiated component's frame should be initialized regarding of its parent's component.
     enum TargetPositionningPolicy {
         SAME_TRANSFORMATION, ///< New component has no parent frame (parent frame is set to nullptr), and its frame is copied from its parent. This is the default.
         NO_TRANSFORMATION,   ///< New component has no parent frame (parent frame is therefore set to nullptr) and transform is Id
         SUBFRAME             ///< New component's parent frame is its parent component
     };
 
-    Q_ENUMS(TargetPositionningPolicy)
+    Q_ENUM(TargetPositionningPolicy)
 
     /**
      * Initializes the window system and constructs a CamiTK application object with argc command line arguments in argv.
@@ -164,12 +183,12 @@ public:
      */
     ///@{
 
-    /// load the filename and returns the corresponding top level Component (return NULL if an error occurs)
+    /// load the filename and returns the corresponding top level Component (returns nullptr if an error occurs)
     /// @note this method opens the filename and created the associated TOP LEVEL component
     /// If you wish to open a subcomponent (not top level then), prefer directly calling its public constructor.
     static Component* open(const QString&);
 
-    /** load a directory and returns the corresponding Component (return NULL if an error occurs)
+    /** load a directory and returns the corresponding Component (returns nullptr if an error occurs)
      * @param dirName the name of the directory to open
      * @param pluginName the name of the plugin to use
      */
@@ -188,7 +207,7 @@ public:
      */
     static bool save(Component* component);
 
-    /// Only the class Component can access to the private members addComponent(..) and removeComponent(..)
+    /// class Component is a friend class: only the class Component can access to the private members addComponent(..) and removeComponent(..)
     friend class Component;
 
     /** get the current application wide list of instantiated top-level Components.
@@ -225,7 +244,6 @@ public:
       * all the previously selected components and clear the list.
       */
     static void clearSelectedComponents();
-
     ///@}
 
     /// @name Action instances management
@@ -255,6 +273,46 @@ public:
     *  @return the number of actions effectively unregistered
     */
     static int unregisterAllActions(ActionExtension*);
+
+    /// set the currently triggered action
+    /// This is used by Action::trigger(..) to refresh all the viewer (the viewers
+    /// interested by the fact an action was triggered can then update what they need to)
+    static void setTriggeredAction(Action* action);
+
+    /// get the currently triggered action
+    static Action* getTriggeredAction();
+    ///@}
+
+    /// @name Viewer management
+    ///@{
+
+    /// get the pointer to a registered viewer given its name
+    static Viewer* getViewer(QString name);
+
+    /// instantiate a new viewer of the given name and given class name (Viewer inheriting class).
+    /// \note this method does not register the viewer automatically. Please call registerViewer(..) to add it
+    /// to the list of registered viewers.
+    static Viewer* getNewViewer(QString name, QString className);
+
+    /// get all the viewers registered in the application (note: the returned ViewerList is garanteed to be sorted by viewer name and to not contain any duplicate)
+    static const ViewerList getViewers();
+
+    /// get all the actions that can be applied on a given component (note: the returned ActionList is garanteed to be sorted by action name and to contain no duplicates)
+    static ViewerList getViewers(Component*);
+
+    /// register a viewer in the viewer list (therefore allowing it to be refreshed by the main window automatically
+    static bool registerViewer(Viewer*);
+
+    /** register all viewers from the given ViewerExtension
+    *  @return the number of viewers effectively registered (in case a viewer's name is already registered
+    * by another viewer extension, it won't be registered a second time)
+    */
+    static int registerAllViewers(ViewerExtension*);
+
+    /** unregister all viewers from the given ViewerExtension
+    *  @return the number of viewer effectively unregistered
+    */
+    static int unregisterAllViewers(ViewerExtension*);
     ///@}
 
     /** @name Main Window management
@@ -274,6 +332,9 @@ public:
     void setMainWindow(MainWindow* mw);
 
     /// get the main window
+    /// \note if there is no MainWindow when this method is first called,
+    /// then this method makes sure there is a MainWindow by and force the creation of
+    /// a default MainWindow (direct instance of the MainWindow class)
     static MainWindow* getMainWindow();
 
     /// refresh the main window (this will call the refresh method of all viewers)
@@ -371,6 +432,8 @@ private:
     /// argv given from command line
     static char** argv;
 
+    static Action* currentAction;
+
     /// @name Recent document management
     ///@{
     /// list of all the remembered recent documents (last opened is last inserted!)
@@ -459,9 +522,11 @@ private:
 
     ///@name Actions pipeline history
     ///@{
-    /// get the history of actions stored.
-    /// * This method follows the "construct on first use" idiom/design-pattern.
-    /** get the singleton stack of actions stored in a pipeline
+    /** get the history of actions stored.
+     *
+     * This methods returns the singleton stack of actions stored in a pipeline as
+     * their are applied during the execution of the application.
+     *
      * This is the private (intern) method.
      * This history is updated (push(), pop()) by addHistoryItem() and removeHistoryItem()
      * This method follows the "construct on first use" idiom/design-pattern.
@@ -474,9 +539,12 @@ private:
 
     /// @name Actions management
     ///@{
-    /** all the registered actions, this is a QMap as actions are mainly sort/compared/process by name (key).
+    /** As actions are mainly sort/compared/process by name, an internal QMap
+     * associates all the contains all the registered actions with their name (key)
      * This is the private (intern) method.
-     * The component extension map is updated by loadExtension, unloadExtension and autoloadExtensions.
+     *
+     * The action extension map is updated by loadExtension, unloadExtension and autoloadExtensions.
+     *
      * This method follows the "construct on first use" idiom/design-pattern.
      * It therefore avoids the infamous "static initialization order fiasco",
      * see http://www.parashift.com/c++-faq/ctors.html
@@ -487,9 +555,29 @@ private:
     static ActionList sort(ActionSet);
     ///@}
 
-    /// Init the application resources by loading the language associated .qm file
-    static void initResources();
+    /// sort an ViewerSet by viewer's name
+    static ViewerList sort(ViewerSet);
+    ///@}
 
+    /// @name viewer management
+    ///@{
+    /** As viewers are mainly sort/compared/process by name, an internal QMap
+     * associates all the registered viewers with their name (key).
+     * This is the private (intern) method.
+     *
+     * This is the private (intern) method.
+     *
+     * The viewer extension map is updated by loadExtension, unloadExtension and autoloadExtensions.
+     *
+     * This method follows the "construct on first use" idiom/design-pattern.
+     * It therefore avoids the infamous "static initialization order fiasco",
+     * see http://www.parashift.com/c++-faq/ctors.html
+     */
+    static QMap<QString, Viewer*>& getViewerMap();
+
+    static ViewerExtension* getViewerExtension(Viewer* viewer);
+
+    ///@}
 
     /// @name Property management
     ///@{
@@ -510,6 +598,9 @@ private:
     static PropertyObject* propertyObject;
     ///@}
 
+    /// Init the application resources by loading the language associated .qml file
+    static void initResources();
+
 protected:
     /**
      * Event filter of this class instance to watch its properties instances.
@@ -522,6 +613,4 @@ protected:
 
 }
 
-Q_DECLARE_METATYPE(camitk::Application::TargetPositionningPolicy)
-
 #endif // CAMITKAPPLICATION_H
