@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -32,6 +32,8 @@
 #include "ComponentExtension.h"
 #include "ActionExtension.h"
 #include "Action.h"
+#include "ViewerExtension.h"
+#include "Viewer.h"
 #include "ObjectController.h"
 #include "ui_Settings.h"
 
@@ -57,6 +59,7 @@ SettingsDialog::SettingsDialog() : QDialog(qApp->activeWindow()) {
     readUserExtensions();
     updateComponentExtensionList();
     updateActionExtensionList();
+    updateViewerExtensionList();
 
     // -- setup the general tab
     myUI->generalTab->setVisible(false);
@@ -98,6 +101,7 @@ void SettingsDialog::readUserExtensions() {
     settings.beginGroup("UserExtensions");
     userComponents = settings.value("components", QVariant(QStringList())).toStringList();
     userActions = settings.value("actions", QVariant(QStringList())).toStringList();
+    userViewers = settings.value("viewers", QVariant(QStringList())).toStringList();
     settings.endGroup();
 }
 
@@ -107,6 +111,7 @@ void SettingsDialog::writeUserExtensions() {
     settings.beginGroup("UserExtensions");
     settings.setValue("components", userComponents);
     settings.setValue("actions", userActions);
+    settings.setValue("viewers", userViewers);
     settings.endGroup();
 }
 
@@ -168,9 +173,8 @@ void SettingsDialog::updateComponentExtensionList() {
 }
 
 
-// ------------------------------ updateComponentExtensionList -------------------------------
+// ------------------------------ updateActionExtensionList -------------------------------
 void SettingsDialog::updateActionExtensionList() {
-    //const QMap<QString, ActionExtension*>& aeMap = ExtensionManager::getActionExtensions();
     const QList<ActionExtension*> aeList = ExtensionManager::getActionExtensionsList();
 
     // remove all the items
@@ -185,7 +189,8 @@ void SettingsDialog::updateActionExtensionList() {
         // create table row
         for (int i = 0; i < ae->getActions().size(); i++) {
             Action* action = ae->getActions().at(i);
-            myUI->actionExtensionList->setItem(actionId, 0, new QTableWidgetItem((userActions.contains(ae->getLocation()) ? QPixmap(":/user") : QPixmap(":/camiTKIcon")), action->getName()));
+            QPixmap actionIcon = action->getIcon();
+            myUI->actionExtensionList->setItem(actionId, 0, new QTableWidgetItem((userActions.contains(ae->getLocation()) ? QPixmap(":/user") : ((actionIcon.isNull()) ? QPixmap(":/camiTKIcon") : actionIcon)), action->getName()));
             myUI->actionExtensionList->setItem(actionId, 1, new QTableWidgetItem(action->getComponent()));
             myUI->actionExtensionList->setItem(actionId, 2, new QTableWidgetItem(ae->getLocation()));
             actionId++;
@@ -197,6 +202,36 @@ void SettingsDialog::updateActionExtensionList() {
     myUI->actionExtensionList->sortByColumn(0, Qt::AscendingOrder);
 }
 
+
+// ------------------------------ updateViewerExtensionList -------------------------------
+void SettingsDialog::updateViewerExtensionList() {
+    const QList<ViewerExtension*> aeList = ExtensionManager::getViewerExtensionsList();
+
+    // remove all the items
+    myUI->viewerExtensionList->clearContents();
+    myUI->viewerExtensionList->setRowCount(Application::getViewers().size());
+    myUI->viewerExtensionList->verticalHeader()->setVisible(false);
+    myUI->viewerExtensionList->setSortingEnabled(false); // temporarily for insertion!
+
+    // loop over all ActionExtensions
+    unsigned int viewerId = 0;
+    foreach (ViewerExtension* ve, aeList) {
+        // create table row
+        for (int i = 0; i < ve->getViewers().size(); i++) {
+            Viewer* viewer = ve->getViewers().at(i);
+            QPixmap viewerIcon = viewer->getIcon();
+            myUI->viewerExtensionList->setItem(viewerId, 0, new QTableWidgetItem((userViewers.contains(ve->getLocation()) ? QPixmap(":/user") : ((viewerIcon.isNull()) ? QPixmap(":/camiTKIcon") : viewerIcon)), viewer->getName()));
+            myUI->viewerExtensionList->setItem(viewerId, 1, new QTableWidgetItem(ve->getName()));
+            myUI->viewerExtensionList->setItem(viewerId, 2, new QTableWidgetItem(ve->getLocation()));
+            viewerId++;
+        }
+    }
+
+    myUI->viewerExtensionList->resizeColumnsToContents();
+    myUI->viewerExtensionList->setSortingEnabled(true);
+    myUI->viewerExtensionList->sortByColumn(0, Qt::AscendingOrder);
+}
+
 // ------------------------------ accept -------------------------------
 void SettingsDialog::accept() {
     if (objectController->isModified()) {
@@ -295,13 +330,14 @@ void SettingsDialog::on_removeComponentExtensionButton_released() {
 
     // removing is allowed by selecting a new component extension
     myUI->removeComponentExtensionButton->setEnabled(false);
+    myUI->componentExtensionDescription->setText("Click on a registered component extension above to display its description.");
 }
 
 // ------------------------------ on_componentExtensionList_cellClicked -------------------------------
 void SettingsDialog::on_componentExtensionList_cellClicked(int row, int column) {
     const ComponentExtension* ce = ExtensionManager::getComponentExtension(myUI->componentExtensionList->item(myUI->componentExtensionList->currentRow(), 0)->text());
     myUI->removeComponentExtensionButton->setEnabled(userComponents.contains(ce->getLocation()));
-    myUI->componentPluginDescription->setText(ce->getDescription());
+    myUI->componentExtensionDescription->setText(ce->getDescription());
 }
 
 // ------------------------------ on_addActionExtensionButton_released -------------------------------
@@ -330,12 +366,14 @@ void SettingsDialog::on_addActionExtensionButton_released() {
     updateActionExtensionList();
 }
 
-
 // ----------------- on_removeActionExtensionButton_released ----------------------------
 void SettingsDialog::on_removeActionExtensionButton_released() {
     // remove the plugin
     QString actionFileName = myUI->actionExtensionList->item(myUI->actionExtensionList->currentRow(), 2)->text();
     ActionList actions = ExtensionManager::getActionExtension(actionFileName)->getActions();
+
+    // If the action extension contains other actions than the one selected, ask the user if this ok
+    // (as removing one action removes the whole extension and the extension might contain more than one action)
     bool reallyDelete = true;
     if (actions.size() > 1) {
         QString actionName = myUI->actionExtensionList->item(myUI->actionExtensionList->currentRow(), 0)->text();
@@ -359,6 +397,7 @@ void SettingsDialog::on_removeActionExtensionButton_released() {
 
         // removing is allowed by selecting a new component extension
         myUI->removeActionExtensionButton->setEnabled(false);
+        myUI->actionExtensionDescription->setText("Click on an registered action above to display its description.");
     }
 
 }
@@ -370,5 +409,74 @@ void SettingsDialog::on_actionExtensionList_cellClicked(int row, int column) {
             (myUI->actionExtensionList->item(myUI->actionExtensionList->currentRow(), 0)->text())->getDescription());
 }
 
+
+// ------------------------------ on_addViewerExtensionButton_released -------------------------------
+void SettingsDialog::on_addViewerExtensionButton_released() {
+    QStringList files = QFileDialog::getOpenFileNames(nullptr, "Add an Viewer Extension", Core::getViewerDirectories().first(), "Viewer Extension (*.so.* *.dll *.dylib)");
+
+    QStringList pluginFile = files;
+    QStringList::Iterator it = pluginFile.begin();
+
+    while (it != pluginFile.end()) {
+        if (!ExtensionManager::loadExtension(ExtensionManager::VIEWER, *it)) {
+            CAMITK_ERROR(tr("ViewerExtension error: cannot load plugin from file:\n\"%1\"\nTry recompile the plugin\n(plugin might be outdated)").arg((*it)))
+        }
+        else {
+            Application::showStatusBarMessage("Plugin " + (*it) + " loaded.");
+            // Add the filename in the application settings: this is a manually loaded plugin, it should therefore
+            // be kept in a list for the next run
+            userViewers += (*it);
+            writeUserExtensions();
+        }
+
+        ++it; //next one!
+    }
+
+    // update the display
+    updateViewerExtensionList();
+}
+
+// ----------------- on_removeViewerExtensionButton_released ----------------------------
+void SettingsDialog::on_removeViewerExtensionButton_released() {
+    // remove the plugin
+    QString viewerFileName = myUI->viewerExtensionList->item(myUI->viewerExtensionList->currentRow(), 2)->text();
+    ViewerList viewers = ExtensionManager::getViewerExtension(viewerFileName)->getViewers();
+
+    // If the viewer extension contains other viewers than the one selected, ask the user if this ok
+    // (as removing one viewer removes the whole extension and the extension might manage more than one viewer)
+    bool reallyDelete = true;
+    if (viewers.size() > 1) {
+        QString viewerName = myUI->viewerExtensionList->item(myUI->viewerExtensionList->currentRow(), 0)->text();
+        QStringList viewerNames;
+        foreach (Viewer* v, viewers) {
+            if (v->getName() != viewerName) {
+                viewerNames << v->getName();
+            }
+        }
+        // CCC Exception: Use a QMessageBox::warning instead of CAMITK_WARNING as the user interaction is required
+        reallyDelete = (QMessageBox::warning(this, "Removing multiple viewers", tr("Removing viewer \"") + viewerName + tr(" \" will also remove the following viewers:<ul><li>") + viewerNames.join("</li><li>") + tr("</li></ul>Are you sure you want to delete all these viewers?"), QMessageBox::Yes | QMessageBox::Cancel) == QMessageBox::Yes);
+    }
+    if (reallyDelete) {
+        ExtensionManager::unloadViewerExtension(viewerFileName);
+        // remove from settings
+        userViewers.removeAll(viewerFileName);
+        writeUserExtensions();
+
+        // update the display
+        updateViewerExtensionList();
+
+        // removing is allowed by selecting a new component extension
+        myUI->removeViewerExtensionButton->setEnabled(false);
+        myUI->viewerExtensionDescription->setText("Click on an registered viewer above to display its description.");
+    }
+}
+
+// ------------------------------ on_viewerExtensionList_cellClicked -------------------------------
+void SettingsDialog::on_viewerExtensionList_cellClicked(int row, int column) {
+    myUI->removeViewerExtensionButton->setEnabled(userViewers.contains(myUI->viewerExtensionList->item(myUI->viewerExtensionList->currentRow(), 2)->text()));
+    myUI->viewerExtensionDescription->setText(Application::getViewer
+            (myUI->viewerExtensionList->item(myUI->viewerExtensionList->currentRow(), 0)->text())->getDescription());
+}
+
 }
 
