@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -38,12 +38,6 @@
 #include <utility>
 
 namespace camitk {
-// --------------------------------------------------------
-// -------------------- static members --------------------
-// --------------------------------------------------------
-QSet<Viewer*> Component::allViewers;
-
-
 // -------------------- constructor --------------------
 Component::Component(QString file, QString n, Representation s) :  myParentNode(nullptr), myFileName(std::move(file)), myService(s), myName(std::move(n)) {
     init();
@@ -52,8 +46,9 @@ Component::Component(QString file, QString n, Representation s) :  myParentNode(
 
 Component::Component(Component* parentComponent, const QString& n, Representation s) : myParentNode(parentComponent), myService(s), myName(n) {
     if (myParentNode == nullptr) {
-        throw AbortException(tr("Inconsistency: cannot instanciate a sub component with a null parent, please use the parent component pointer as the first parameter of the constructor or use the top-level Component constructor.").toStdString());
+        throw AbortException(tr("Inconsistency: cannot instantiate a sub component with a null parent, please use the parent component pointer as the first parameter of the constructor or use the top-level Component constructor.").toStdString());
     }
+
     init();
     myFileName = n;
     // register as a parent child
@@ -80,30 +75,53 @@ Component::~Component() {
     if (myGeometry) {
         delete myGeometry;
     }
+
     myGeometry = nullptr;
 
     if (mySlice) {
         delete mySlice;
     }
+
     mySlice = nullptr;
 
     // delete Frame
     if (myFrame) {
         delete myFrame;
     }
+
     myFrame = nullptr;
 
     // remove from selection
     if (isSelectedFlag) {
-        setSelected(false, false);
+        // Beware that this calls the method 'Component::setSelected' (as define in this class)
+        // which bypass any redefined setSelected method in a derived class
+        Component::setSelected(false, false);
     }
 
     // delete all properties
-    foreach (Property* prop, propertyMap.values()) {
+    for (auto prop : qAsConst(propertyMap)) {
         delete prop;
     }
+
     propertyMap.clear();
 
+    if (actionsMenu != nullptr) {
+        actionsMenu->clear();
+        // deleteLater() is required instead of a simple delete to avoid crash
+        // Explanation:
+        // This menu is a context menu of the Explorer Viewer.
+        // At one point the user can right-click on this Component node representation in the explorer,
+        // which will run the exec() method of this actionsMenu.
+        // Now, during this exec() call, if the user selects the "Close" action, the processor
+        // will arrive in this destructor.
+        // If we leave a classic "delete" here, this will delete the currently opened
+        // context menu and free all the memory, while it is still being currently in its exec()
+        // This might crash the event loop (which runs in another thread) as it is still using the object.
+        // deleteLater() is safe as it will handle by the event loop later on.
+        actionsMenu->deleteLater();
+        actionsMenu = nullptr;
+    }
+
     CAMITK_TRACE(tr("Deleted"))
 }
 
@@ -113,6 +131,7 @@ void Component::init() {
     Application::addComponent(this);
 
     modifiedFlag = false;
+    interfaceNodeModifiedFlag = false;
     childrenComponent.clear();
 
     // initialize representation
@@ -120,6 +139,7 @@ void Component::init() {
     mySlice = nullptr;
     myFrame = nullptr;
     actionsMenu = nullptr;
+    visibilityMenu = nullptr;
 
     // create the Frame (does not depend on any representation, should be created for any Component)
     myFrame = new Frame();
@@ -131,33 +151,7 @@ void Component::init() {
     installEventFilter(this);
 
     // by default, the 1st PropertyExplorer tab is displayed
-    this->setIndexOfPropertyExplorerTab(0);
-
-    CAMITK_TRACE(tr("Initialized"))
-}
-
-
-// -------------------- isInstanceOf --------------------
-bool Component::isInstanceOf(QString className) const {
-    if (getHierarchy().contains(className)) {
-        return true;
-    }
-    else {
-        return false;
-    }
-}
-
-// -------------------- getHierarchy --------------------
-QStringList Component::getHierarchy() const {
-    const QMetaObject* qmetaObject = metaObject();
-    QStringList classnameList;
-    while (qmetaObject && !QString(qmetaObject->className()).contains("InterfaceProperty")) {
-        classnameList.append(QString(qmetaObject->className()).remove(0, QString(qmetaObject->className()).lastIndexOf(':') + 1));
-        if (qmetaObject->superClass()) {
-            qmetaObject = qmetaObject->superClass();
-        }
-    }
-    return classnameList;
+    setIndexOfPropertyExplorerTab(0);
 }
 
 // ---------------------- event ----------------------------
@@ -166,7 +160,7 @@ bool Component::event(QEvent* e) {
         e->accept();
         auto* chev = dynamic_cast<QDynamicPropertyChangeEvent*>(e);
 
-        if (! chev) {
+        if (!chev) {
             return false;
         }
 
@@ -182,6 +176,308 @@ bool Component::event(QEvent* e) {
     return QObject::event(e);
 }
 
+// -------------------- isTopLevel --------------------
+bool Component::isTopLevel() const {
+    return (myParentNode == nullptr);
+}
+
+
+// -------------------- getParentComponent --------------------
+Component* Component::getParentComponent() {
+    return (Component*)myParentNode;
+}
+
+// -------------------- getTopLevelComponent --------------------
+Component* Component::getTopLevelComponent() {
+    if (!isTopLevel()) {
+        return getParentComponent()->getTopLevelComponent();
+    }
+    else {
+        return this;
+    }
+}
+
+// -------------------- setVisibility --------------------
+void Component::setVisibility(QString viewerName, bool b) {
+    // first check if viewer is registered
+    Viewer* v = Application::getViewer(viewerName);
+
+    // only insert registered viewers
+    if (v != nullptr) {
+        auto it = myViewers.find(viewerName);
+
+        if (it == myViewers.end()) {
+            // insert the new viewer with the corresponding boolean
+            myViewers.insert(viewerName, b);
+        }
+        else {
+            it.value() = b;
+        }
+    }
+
+}
+
+void Component::setVisibility(Viewer* v, bool b) {
+    CAMITK_WARNING(tr("setVisibility(Viewer*,bool) method is now deprecated and will be removed from CamiTK API, please use setVisibility(QString,bool) instead"))
+
+    // only insert non-null and registered viewers
+    if (v != nullptr && Application::getViewer(v->getName()) != nullptr) {
+        auto it = myViewers.find(v->getName());
+
+        if (it == myViewers.end()) {
+            // insert the new viewer with the corresponding boolean
+            myViewers.insert(v->getName(), b);
+        }
+        else {
+            it.value() = b;
+        }
+    }
+}
+
+// -------------------- getVisibility --------------------
+bool Component::getVisibility(QString viewerName) const {
+    auto it = myViewers.constFind(viewerName);
+
+    if (it == myViewers.end()) {
+        return false;
+    }
+    else {
+        return it.value();
+    }
+}
+
+bool Component::getVisibility(Viewer* v) const {
+    CAMITK_WARNING(tr("getVisibility(Viewer*) method is now deprecated and will be removed from CamiTK API, please use getVisibility(QString) instead"))
+
+    auto it = myViewers.constFind(v->getName());
+
+    if (v == nullptr || it == myViewers.end()) {
+        return false;
+    }
+    else {
+        return it.value();
+    }
+}
+
+// -------------------- refresh --------------------
+void Component::refresh() {
+    for (auto it = myViewers.begin(); it != myViewers.end(); it++) {
+        if (it.value()) {
+            Application::getViewer(it.key())->refresh();
+        }
+    }
+    setNodeModified(false);
+}
+
+// -------------------- refreshInterfaceNode --------------------
+void Component::refreshInterfaceNode() {
+    CAMITK_WARNING(tr("refreshInterfaceNode() is not required anymore. Everything should now be done transparently (if it does not work, use setNodeModified(true) to force a specific InterfaceNode refresh instead)"))
+    refresh();
+}
+
+// -------------------- getService --------------------
+Component::Representation Component::getRepresentation() const {
+    static bool checked = false; // only check once for each Component
+
+    if (!checked) {
+        checked = true;
+        QString representationString;
+        QString instantiatedMember;
+        QString shouldInstantiateMember;
+        bool instantiationError = false;
+
+        // check the service
+        switch (myService) {
+            case GEOMETRY:
+                representationString = "GEOMETRY";
+                shouldInstantiateMember = "myGeometry";
+                instantiationError = (myGeometry == nullptr);
+                break;
+
+            case SLICE:
+                representationString = "SLICE";
+                shouldInstantiateMember = "mySlice";
+                instantiationError = (mySlice == nullptr);
+                break;
+
+            case NO_REPRESENTATION:
+                instantiatedMember = (myGeometry != nullptr) ? "myGeometry" : (mySlice != nullptr) ? "mySlice" : "<<INTERNAL_ERROR>>";
+                instantiationError = (mySlice != nullptr || myGeometry != nullptr);
+                break;
+        }
+
+        if (instantiationError) {
+            const QMetaObject* qmetaObject = metaObject();
+            QString classname = qmetaObject->className();
+
+            if (myService != NO_REPRESENTATION) {
+                // the Component has to instantiate
+                CAMITK_ERROR(tr("Component class \"%1\" implements service %2 but does not instantiate it!\n"
+                                "\tPlease instantiate %3 in initRepresentation() or change the implemented representation in the constructor.").arg(classname, representationString, shouldInstantiateMember))
+            }
+            else {
+                // no representation should be instantiated!
+                CAMITK_ERROR(tr("Component class \"%1\" has no implemented representation (NO_REPRESENTATION) but %2 is instantiated. Please do not instantiate %3 in initRepresentation() or change the implemented service in the constructor.").arg(classname, instantiatedMember, instantiatedMember))
+            }
+        }
+    }
+
+    return myService;
+}
+
+// ---------------- actionLessThan ----------------
+extern bool actionLessThan(const camitk::Action* a1, const camitk::Action* a2);
+
+// -------------------- getActionMenu -----------------------
+QMenu* Component::getActionMenu() {
+
+    ActionList allActions = Application::getActions(this);
+    if (actionsMenu == nullptr) {
+        actionsMenu = new QMenu();
+
+        //-- add all actions sorted by family
+        QMap<QString, ActionSet> familyMap;
+
+        // group actions by family
+        for (Action* action : qAsConst(allActions)) {
+            familyMap[action->getFamily().toLower()].insert(action);
+        }
+
+        //-- create the "File" and "View" submenu first
+        QMenu* fileMenu = actionsMenu->addMenu("File");
+        QMenu* viewMenu = actionsMenu->addMenu("View");
+
+        //-- create one sub menu per family (unless there is only one action)
+        for (const QSet<Action*>& familySet : qAsConst(familyMap)) {
+            // sort actions by name
+            ActionList familyList = familySet.toList();
+            std::sort(familyList.begin(), familyList.end(), actionLessThan);
+
+            if (familyList.size() >= 1) {
+                QMenu* familyMenu;
+                if (familyList.first()->getFamily() == "File") {
+                    familyMenu = fileMenu;
+                }
+                else {
+                    if (familyList.first()->getFamily() == "View") {
+                        familyMenu = viewMenu;
+                    }
+                    else {
+                        // this is a new family
+                        familyMenu = actionsMenu->addMenu(familyList.first()->getFamily());
+                    }
+                }
+
+                foreach (Action* action, familyList) {
+                    // ownership not taken here
+                    familyMenu->addAction(action->getQAction(this));
+                }
+            }
+            else {
+                // ownership not taken here
+                actionsMenu->addAction(familyList.first()->getQAction(this));
+            }
+        }
+
+        // add the visibility submenu
+        visibilityMenu = viewMenu->addMenu("Viewer Visibility");
+    }
+
+    //-- update viewer visibility and all other actions that are checkable
+    for (Action* a : qAsConst(allActions)) {
+        a->getQAction(this);
+    }
+    visibilityMenu->clear();
+    ViewerList allViewers = Application::getViewers(this);
+
+    for (Viewer* viewer : qAsConst(allViewers)) {
+        // show only embedded or stacked viewers
+        if (viewer->getType() != Viewer::DOCKED) {
+            // viewerAction is own by visibilityMenu, visibilityMenu.clear() will delete it
+            QAction* visibilityAction = visibilityMenu->addAction(viewer->getName());
+            visibilityAction->setCheckable(true);
+            visibilityAction->setChecked(getVisibility(viewer->getName()));
+            visibilityAction->setIcon(viewer->getIcon());
+            QString tipString = "Set the visibility of this component in the \"" + viewer->getName() + "\" viewer\n(note that it might not have effect depending on the viewer)";
+            visibilityAction->setStatusTip(tr(tipString.toStdString().c_str()));
+            visibilityAction->setWhatsThis(tr(tipString.toStdString().c_str()));
+            // add the toggle action slot using C++11 lambda so that everything is contained inside viewMenu
+            connect(visibilityAction, &QAction::toggled, [ this, viewer ](bool visible) {
+                setVisibility(viewer->getName(), visible);
+                Application::refresh();
+            });
+        }
+    }
+
+    // no menu if no action
+    if (actionsMenu->actions().size() == 0) {
+        delete actionsMenu;
+        actionsMenu = nullptr;
+    }
+
+    return actionsMenu;
+}
+
+// -------------------- setSelected --------------------
+void Component::setSelected(const bool b, const bool recursive) {
+    isSelectedFlag = b;
+
+    // maintain the children selection state as well
+    if (recursive) {
+        foreach (Component* child, childrenComponent) {
+            child->setSelected(b, recursive);
+        }
+    }
+
+    //-- Now add to global selection
+    // do that only in the end, so that the last selected will be this Component (which is the one called first!)
+    Application::setSelected(this, b);
+}
+
+// -------------------- getFileName --------------------
+const QString Component::getFileName() const {
+    return myFileName;
+}
+
+// -------------------- getFileName --------------------
+void Component::setFileName(const QString& fName) {
+    myFileName = fName;
+}
+
+
+
+
+// -----------------------------------------------------------------------
+//
+//                        InterfaceProperty methods
+//
+// -----------------------------------------------------------------------
+
+// -------------------- getHierarchy --------------------
+QStringList Component::getHierarchy() const {
+    const QMetaObject* qmetaObject = metaObject();
+    QStringList classnameList;
+
+    while (qmetaObject && !QString(qmetaObject->className()).contains("InterfaceProperty")) {
+        classnameList.append(QString(qmetaObject->className()).remove(0, QString(qmetaObject->className()).lastIndexOf(':') + 1));
+
+        if (qmetaObject->superClass()) {
+            qmetaObject = qmetaObject->superClass();
+        }
+    }
+
+    return classnameList;
+}
+
+// -------------------- isInstanceOf --------------------
+bool Component::isInstanceOf(QString className) const {
+    if (getHierarchy().contains(className)) {
+        return true;
+    }
+    else {
+        return false;
+    }
+}
 
 // ---------------------- updateProperty ----------------------------
 void Component::updateProperty(QString name, QVariant value) {
@@ -209,9 +505,59 @@ bool Component::addProperty(Property* prop) {
     return returnStatus;
 }
 
-// -------------------- isTopLevel --------------------
-bool Component::isTopLevel() const {
-    return (myParentNode == nullptr);
+
+
+
+// -----------------------------------------------------------------------
+//
+//                        InterfaceNode methods
+//
+// -----------------------------------------------------------------------
+
+
+// -------------------- addChild --------------------
+void Component::addChild(InterfaceNode* childNode) {
+    attachChild(childNode);
+    childNode->setParent(this);
+    setNodeModified(true);
+}
+
+
+// -------------------- removeChild --------------------
+void Component::removeChild(InterfaceNode* childNode) {
+    if (childrenComponent.removeAll((Component*)childNode)) {
+        // remove myself from the childNode's parent item list
+        childNode->setParent(nullptr);
+        setNodeModified(true);
+    }
+}
+
+// -------------------- deleteChildren --------------------
+void Component::deleteChildren() {
+    // calling clear() on a QSet does not delete the instances
+    // (the destuctor of points is not called, the Component are not deleted from the memory)
+    foreach (Component* childComp, childrenComponent) {
+        if (Application::isAlive(childComp)) {
+            delete childComp;
+        }
+    }
+
+    childrenComponent.clear();
+    setNodeModified(true);
+}
+
+// -------------------- attachChild --------------------
+void Component::attachChild(InterfaceNode* childNode) {
+    auto* comp = dynamic_cast<Component*>(childNode);
+
+    if (comp != nullptr) {
+        // add a sub item
+        if (!childrenComponent.contains(comp)) {
+            // add it to the list
+            childrenComponent.append(comp);
+        }
+        setNodeModified(true);
+    }
 }
 
 // -------------------- setParent ------------------
@@ -232,155 +578,14 @@ void Component::setParent(InterfaceNode* pes) {
     }
 
     myParentNode = pes;
-}
-
-// -------------------- getParentComponent --------------------
-Component* Component::getParentComponent() {
-    return (Component*)myParentNode;
-}
-
-// -------------------- getTopLevelComponent --------------------
-Component* Component::getTopLevelComponent() {
-    if (!isTopLevel()) {
-        return getParentComponent()->getTopLevelComponent();
-    }
-    else {
-        return this;
-    }
-}
-
-// -------------------- setVisibility --------------------
-void Component::setVisibility(Viewer* v, bool b) {
-    QMap<Viewer*, bool>::iterator it = myViewers.find(v);
-    if (it == myViewers.end()) {
-        // insert the new viewer with the corresponding boolean
-        myViewers.insert(v, b);
-    }
-    else {
-        it.value() = b;
-    }
-}
-
-// -------------------- getVisibility --------------------
-bool Component::getVisibility(Viewer* v) const {
-    QMap<Viewer*, bool>::const_iterator it = myViewers.constFind(v);
-    if (it == myViewers.end()) {
-        return false;
-    }
-    else {
-        return it.value();
-    }
-}
-
-// -------------------- refresh --------------------
-void Component::refresh() const {
-    for (QMap<Viewer*, bool>::const_iterator it = myViewers.begin(); it != myViewers.end(); it++) {
-        if (it.value()) {
-            it.key()->refresh();
-        }
-    }
-}
-
-// -------------------- refreshInterfaceNode --------------------
-void Component::refreshInterfaceNode() {
-    for (QMap<Viewer*, bool>::const_iterator it = myViewers.begin(); it != myViewers.end(); it++) {
-        if (it.value()) {
-            it.key()->refreshInterfaceNode(this);
-        }
-    }
+    setNodeModified(true);
 }
 
 
 
-// -------------------- getService --------------------
-Component::Representation Component::getRepresentation() const {
-    static bool checked = false; // only check once for each Component
-    if (!checked) {
-        checked = true;
-        QString representationString;
-        QString instantiatedMember;
-        QString shouldInstanciateMember;
-        bool instanciationError = false;
 
-        // check the service
-        switch (myService) {
-            case GEOMETRY:
-                representationString = "GEOMETRY";
-                shouldInstanciateMember = "myGeometry";
-                instanciationError = (myGeometry == nullptr);
-                break;
-            case SLICE:
-                representationString = "SLICE";
-                shouldInstanciateMember = "mySlice";
-                instanciationError = (mySlice == nullptr);
-                break;
-            case NO_REPRESENTATION:
-                instantiatedMember = (myGeometry != nullptr) ? "myGeometry" : (mySlice != nullptr) ? "mySlice" : "<<INTERNAL_ERROR>>";
-                instanciationError = (mySlice != nullptr || myGeometry != nullptr);
-                break;
-        }
-        if (instanciationError) {
-            const QMetaObject* qmetaObject = metaObject();
-            QString classname = qmetaObject->className();
-            if (myService != NO_REPRESENTATION) {
-                // the Component has to instanciate
-                CAMITK_ERROR(tr("Component class \"%1\" implements service %2 but does not instanciate it!\n"
-                                "\tPlease instanciate %3 in initRepresentation() or change the implemented representation in the constructor.").arg(classname, representationString, shouldInstanciateMember))
-            }
-            else {
-                // no representation should be instantiated!
-                CAMITK_ERROR(tr("Component class \"%1\" has no implemented representation (NO_REPRESENTATION) but %2 is instantiated. Please do not instantiate %3 in initRepresentation() or change the implemented service in the constructor.").arg(classname, instantiatedMember, instantiatedMember))
-            }
-        }
-    }
-    return myService;
-}
 
-// ---------------- actionLessThan ----------------
-extern bool actionLessThan(const camitk::Action* a1, const camitk::Action* a2);
 
-// -------------------- getActionMenu -----------------------
-QMenu* Component::getActionMenu() {
-
-    if (actionsMenu == nullptr) {
-        actionsMenu = new QMenu();
-
-        //-- add all actions sorted by family
-        ActionList allActions = Application::getActions(this);
-        QMap<QString, ActionSet*> familyMap;
-        foreach (Action* action, allActions) {
-            ActionSet* familySet = familyMap.value(action->getFamily().toLower());
-            if (!familySet) {
-                familySet = new ActionSet;
-                familyMap.insert(action->getFamily().toLower(), familySet);
-            }
-            familySet->insert(action);
-        }
-
-        //-- create one sub menu per family (unless there is only one action)
-        foreach (ActionSet* familySet, familyMap.values()) {
-            // sort actions by name
-            ActionList familyList = familySet->toList();
-            qSort(familyList.begin(), familyList.end(), actionLessThan);
-            if (familyList.size() >= 1) {
-                QMenu* familyMenu = actionsMenu->addMenu(familyList.first()->getFamily());
-                foreach (Action* action, familyList) {
-                    familyMenu->addAction(action->getQAction());
-                }
-            }
-            else {
-                actionsMenu->addAction(familyList.first()->getQAction());
-            }
-        }
-
-        // no menu if no action
-        if (actionsMenu->actions().size() == 0) {
-            delete actionsMenu;
-            actionsMenu = nullptr;
-        }
-    }
-    return actionsMenu;
-}
 
 
 // -----------------------------------------------------------------------
@@ -401,9 +606,11 @@ const InterfaceGeometry::RenderingModes Component::getRenderingModes() const {
         // else return the added rendering mode of all children
         if (childrenComponent.size() > 0) {
             InterfaceGeometry::RenderingModes m = InterfaceGeometry::None;
+
             foreach (Component* childComponent, childrenComponent) {
                 m |= childComponent->getRenderingModes();
             }
+
             return m;
         }
     }
@@ -422,11 +629,13 @@ void Component::getActorColor(InterfaceGeometry::RenderingModes m, double d[4])
         // return the firts existing Color in my children
         int i = 0;
         bool found = false;
+
         while (i < childrenComponent.size() && !found) {
             childrenComponent[i]->getActorColor(m, d);
             found = (d[0] != 0.0 || d[1] != 0.0 || d[2] != 0.0 || d[3] != 0.0);
             i++;
         }
+
         if (!found) {
             for (unsigned int j = 0; j < 4; j++) {
                 d[j] = 0.0;
@@ -441,17 +650,20 @@ void Component::getBounds(double* bounds) {
     else {
         bounds[0] = bounds[2] = bounds[4] = 0.0;
         bounds[1] = bounds[3] = bounds[5] = 1.0;
+
         // compute bounds using the children's
         foreach (Component* childComponent, childrenComponent) {
             double childBounds[6]; //xmin,xmax, ymin,ymax, zmin,zmax
             // get child bounds
             childComponent->getBounds(childBounds);
+
             // check compared to global bound
             for (int i = 0; i < 3; i++) {
                 // compare min
                 if (bounds[i * 2] > childBounds[i * 2]) {
                     bounds[i * 2] = childBounds[i * 2];
                 }
+
                 // compare max
                 if (bounds[i * 2 + 1] < childBounds[i * 2 + 1]) {
                     bounds[i * 2 + 1] = childBounds[i * 2 + 1];
@@ -470,17 +682,25 @@ double Component::getBoundingRadius() {
     else {
         // compute bounding radius using the children's
         double radius = 0.0;
+
         foreach (Component* childComponent, childrenComponent) {
             double childRadius = childComponent->getBoundingRadius();
+
             if (childRadius > radius) {
                 radius = childRadius;
             }
         }
+
         return radius;
     }
 }
 
-
+// -------------------- setGlyphType --------------------
+void Component::setGlyphType(const camitk::InterfaceGeometry::GlyphTypes type, const double size) {
+    if (myGeometry) {
+        myGeometry->setGlyphType(type, size);
+    }
+}
 
 
 
@@ -512,76 +732,10 @@ int Component::getSlice() const {
     }
 }
 
-// -------------------- attachChild --------------------
-void Component::attachChild(InterfaceNode* childNode) {
-    auto* comp = dynamic_cast<Component*>(childNode);
-    if (comp != nullptr) {
-        // add a sub item
-        if (!childrenComponent.contains(comp)) {
-            // add it to the list
-            childrenComponent.append(comp);
-        }
-    }
-}
-
-// -------------------- addChild --------------------
-void Component::addChild(InterfaceNode* childNode) {
-    attachChild(childNode);
-    childNode->setParent(this);
-}
 
 
-// -------------------- setSelected --------------------
-void Component::setSelected(const bool b, const bool recursive) {
-    isSelectedFlag = b;
-    // maintain the children selection state as well
-    if (recursive) {
-        foreach (Component* child, childrenComponent) {
-            child->setSelected(b, recursive);
-        }
-    }
 
-    //-- Now add to global selection
-    // do that only in the end, so that the last selected will be this Component (which is the one called first!)
-    Application::setSelected(this, b);
-}
 
-// -------------------- getFileName --------------------
-const QString Component::getFileName() const {
-    return myFileName;
-}
-
-// -------------------- getFileName --------------------
-void Component::setFileName(const QString& fName) {
-    myFileName = fName;
-}
-
-// -------------------- removeChild --------------------
-void Component::removeChild(InterfaceNode* childNode) {
-    if (childrenComponent.removeAll((Component*)childNode)) {
-        // remove myself from the childNode's parent item list
-        childNode->setParent(nullptr);
-    }
-}
-
-// -------------------- deleteChildren --------------------
-void Component::deleteChildren() {
-    // calling clear() on a QSet does not delete the instances
-    // (the destuctor of points is not called, the Component are not deleted from the memory)
-    foreach (Component* childComp, childrenComponent) {
-        if (Application::isAlive(childComp)) {
-            delete childComp;
-        }
-    }
-    childrenComponent.clear();
-}
-
-// -------------------- setGlyphType --------------------
-void Component::setGlyphType(const camitk::InterfaceGeometry::GlyphTypes type, const double size) {
-    if (myGeometry) {
-        myGeometry->setGlyphType(type, size);
-    }
-}
 
 // -----------------------------------------------------------------------
 //
@@ -626,7 +780,10 @@ const QVector<InterfaceFrame*>& Component::getChildrenFrame() const {
     if (myFrame == nullptr) {
         CAMITK_ERROR(tr("myFrame is not instantiated."))
         // error / undefined behaviour
+        // this will generate a crash
+        throw std::runtime_error("Cannot call getChildrenFrame() on this type of Component.");
     }
+
     return myFrame->getChildrenFrame();
 }
 
@@ -636,6 +793,7 @@ const vtkSmartPointer<vtkTransform> Component::getTransformFromWorld() const {
         CAMITK_ERROR(tr("myFrame is not instantiated."))
         return nullptr;
     }
+
     return myFrame->getTransformFromWorld();
 }
 
@@ -645,6 +803,7 @@ const vtkSmartPointer<vtkTransform> Component::getTransform() const {
         CAMITK_ERROR(tr("myFrame is not instantiated."))
         return nullptr;
     }
+
     return myFrame->getTransform();
 }
 
@@ -654,6 +813,7 @@ const vtkSmartPointer<vtkTransform> Component::getTransformFromFrame(InterfaceFr
         CAMITK_ERROR(tr("myFrame is not instantiated."))
         return nullptr;
     }
+
     return myFrame->getTransformFromFrame(frame);
 }
 
@@ -669,16 +829,39 @@ vtkSmartPointer<vtkAxesActor> Component::getFrameAxisActor() {
     }
 }
 
+// -------------------- setFrameVisibility --------------------
+void Component::setFrameVisibility(Viewer* v, bool visibility) {
+    if (myFrame == nullptr) {
+        CAMITK_ERROR(tr("myFrame is not instantiated."))
+    }
+    else {
+        CAMITK_WARNING(tr("setFrameVisibility(Viewer*,bool) method is now deprecated and will be removed from CamiTK API, please use setFrameVisibility(QString,bool) instead"))
+        myFrame->setFrameVisibility(v->getName(), visibility);
+    }
+}
+
 // -------------------- getFrameVisibility --------------------
+bool Component::getFrameVisibility(QString viewerName) const {
+    if (myFrame == nullptr) {
+        CAMITK_ERROR(tr("myFrame is not instantiated."))
+        return false;
+    }
+    else {
+        return myFrame->getFrameVisibility(viewerName);
+    }
+}
+
 bool Component::getFrameVisibility(Viewer* viewer) const {
     if (myFrame == nullptr) {
         CAMITK_ERROR(tr("myFrame is not instantiated."))
         return false;
     }
     else {
-        return myFrame->getFrameVisibility(viewer);
+        CAMITK_WARNING(tr("getFrameVisibility(Viewer*) method is now deprecated and will be removed from CamiTK API, please use getFrameVisibility(QString) instead"))
+        return myFrame->getFrameVisibility(viewer->getName());
     }
 }
 
 }
 
+
