@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -94,7 +94,7 @@ public:
      */
     MeshComponent(const QString& file);
 
-    /** Creates a top-level MeshComponent from a vtkPointSet (and instanciate its 3D representation).
+    /** Creates a top-level MeshComponent from a vtkPointSet (and instantiate its 3D representation).
      *
      * \note the filename is set to the empty string "".
      *
@@ -105,7 +105,7 @@ public:
      */
     MeshComponent(vtkSmartPointer<vtkPointSet> aPointSet, const QString& name);
 
-    /** Creates a MeshComponent as a sub component of another Component using a vtkPointSet (and instanciate its 3D representation).
+    /** Creates a MeshComponent as a sub component of another Component using a vtkPointSet (and instantiate its 3D representation).
      *
      * \note if aPointSet is NULL, the representation is not initialized, it is the responsability of the subclass to initialize it later
      *
@@ -131,7 +131,7 @@ public:
     vtkIdType getPickedCellId();
 
     /// update property: no specific properties to manage, this is needed to avoid console warnings.
-    void updateProperty(QString, QVariant) {}
+    void updateProperty(QString, QVariant) override {}
 
     /// there is more than one property widgets (to view as tabs in the property explorer)
     unsigned int getNumberOfPropertyWidget() override;
@@ -186,7 +186,7 @@ public:
     /**
      * @brief Get a selection from its name.
      *
-     * @TODO This method should return a const ref, so that external code cannot modify it
+     * TODO This method should return a const ref, so that external code cannot modify it
      *
      * @param name name of the selection
      * @return the selection node
@@ -196,7 +196,7 @@ public:
     /**
      * @brief Get a selection from its index in the list.
      *
-     * @TODO This method should return a const ref, so that external code cannot modify it
+     * TODO This method should return a const ref, so that external code cannot modify it
      *
      * @param index index of the selection
      * @return the selection node
@@ -365,16 +365,16 @@ public:
 protected:
 
     /// build the instance of Geometry from the given vtkPointSet
-    virtual void initRepresentation(vtkSmartPointer<vtkPointSet>);
+    void initRepresentation(vtkSmartPointer<vtkPointSet>);
 
     /// initialize selections
-    virtual void initSelection();
+    void initSelection();
 
     /// initialize data
-    virtual void initData();
+    void initData();
 
     /// create and initialize dynamic properties
-    virtual void initDynamicProperties();
+    void initDynamicProperties();
 
 protected slots:
 
@@ -396,7 +396,7 @@ protected slots:
 private:
 
     /// the concrete building of the 3D objects (Slice/Geometry): none in this case, everything is done by initRepresentation(vtkPointSet)
-    void initRepresentation() {}
+    void initRepresentation() override final {}
 
     /// initialisation of the mesh component members
     void init();
