@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -344,7 +344,6 @@ void ObjectControllerPrivate::updateDynamicProperties(const QObject* edited) {
     QList<QByteArray> dynProp = edited->dynamicPropertyNames();
 
     for (int i = 0; i < dynProp.size(); ++i) {
-        QString pName = QString(dynProp.at(i));
         int type = edited->property(dynProp.at(i)).userType();
         QtVariantProperty* subProperty = m_indexToDynamicProperty[i];
 
@@ -592,12 +591,12 @@ QObject* ObjectController::object() const {
 // CAMITK ADDED
 void ObjectController::setAutoUpdateProperty(bool autoUpdate) {
     if (autoUpdate) {
-        connect(d_ptr->m_manager, SIGNAL(valueChanged(QtProperty*, const QVariant&)),
-                this, SLOT(valueChanged(QtProperty*, const QVariant&)));
+        connect(d_ptr->m_manager, SIGNAL(valueChanged(QtProperty*, QVariant)),
+                this, SLOT(valueChanged(QtProperty*, QVariant)));
     }
     else {
-        connect(d_ptr->m_manager, SIGNAL(valueChanged(QtProperty*, const QVariant&)),
-                this, SLOT(saveChange(QtProperty*, const QVariant&)));
+        connect(d_ptr->m_manager, SIGNAL(valueChanged(QtProperty*, QVariant)),
+                this, SLOT(saveChange(QtProperty*, QVariant)));
     }
 }
 
