@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -201,18 +201,13 @@ namespace camitk {
       - register the enum type name to the property using the Property::setEnumTypeName
 
     \note
-    If your Qt version is old (version < 5.5), then use Q_ENUMS instead of Q_ENUM and
-    don't forget to register the enum as a new type using Q_DECLARE_METATYPE outside your class declaration
-    as well.
-    Using Qt >=5.5 and Q_ENUM is recommended.
-
     Anywhere in your code use the property(..).toInt() method to get the classical enum value of the property,
     and use Property::getEnumValueAsString() to get the enum value as a string.
-    If you declared your enum using Q_ENUM (instead of deprecated Q_ENUMS), you can also use directly
+    If you declared your enum using Q_ENUM (Qt>=5.5), you can also use directly
     QMetaEnum::fromType to retrieve the enum value as strings (see below).
 
-    You can the enum value names in the GUI using the "enumNames" attributes.
-    There is also a way to automatically build nicer enumNames (\s
+    You can change the enum value names in the GUI using the "enumNames" attributes.
+    There is also a way to automatically build nicer enumNames (see below).
 
     Enum icons might be set using Property::setEnumIcons.
 
@@ -228,19 +223,14 @@ namespace camitk {
                 PossibleValue2
             };
 
-            // register the enum in Qt >= 5.5
+            // register the enum (Qt >= 5.5)
             Q_ENUM(MyEnum)
-            // Note: use the following line instead in Qt < 5.5 and don't forge the Q_DECLARE_METATYPE (see below)
-            // Q_ENUMS(MyEnum)
 
             ...
             private:
             // the enum property
             Property *enumProp;
         };
-
-        // Note: you also need to register the equivalent meta type for the new enum if Qt < 5.5
-        // Q_DECLARE_METATYPE(MyAction::MyEnum);
     \endcode
 
     And then in the code:
@@ -276,12 +266,11 @@ namespace camitk {
             // get the value as a QString (either "PossibleValue1" or "PossibleValue2", beware: this is different from the GUI names)
             QString enumPropAsString = enumProp->getEnumValueAsString(this);
 
-            // OR if you use the recommended Q_ENUM declaration:
+            // Or thanks to Q_ENUM declaration:
             QMetaEnum metaEnum = QMetaEnum::fromType<EnumerationExample>();
             QString enumPropAsStringDirect = metaEnum.valueToKey(enumPropCurrentValue);
     \endcode
 
-    \endcode
     \note
     This is not exactly a decorator pattern, as the Property class is not abstract. The Qt
     Meta Object is still held by the QtObject inherited class (e.g. Component or Action).
