@@ -2,7 +2,7 @@
  * $CAMITK_LICENCE_BEGIN$
  *
  * CamiTK - Computer Assisted Medical Intervention ToolKit
- * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
+ * (c) 2001-2021 Univ. Grenoble Alpes, CNRS, Grenoble INP, TIMC, 38000 Grenoble, France
  *
  * Visit http://camitk.imag.fr for more information
  *
@@ -32,13 +32,14 @@
 #include "Component.h"
 #include "Viewer.h"
 #include "RendererWidget.h"
+#include "InteractiveViewerFrame.h"
 
 //-- QT stuff
-#include <QFrame>
 #include <QPushButton>
 #include <QWidgetAction>
 #include <QComboBox>
 class QToolBar;
+class QFrame;
 
 //-- vtk stuff
 #include <vtkType.h>
@@ -65,19 +66,6 @@ class Property;
 
 using vtkSmartPointerCamera = vtkSmartPointer<vtkCamera>;
 
-/// InteractiveViewerFrame is just a QFrame that delegates all key events to its InteractiveViewer
-class InteractiveViewerFrame : public QFrame {
-public:
-    /// default constructor
-    InteractiveViewerFrame(QWidget* parent, InteractiveViewer* s3D) : QFrame(parent), myInteractiveViewer(s3D) {}
-
-    /// Handle keyboard events in the scene frame, just send everything to InteractiveViewer!
-    void keyPressEvent(QKeyEvent*) override;
-
-protected:
-    InteractiveViewer* myInteractiveViewer;
-};
-
 /**
   * @ingroup group_sdk_libraries_core_viewer
   *
@@ -92,22 +80,6 @@ protected:
   * The keyboard events are all managed in InteractiveViewer as well.
   * Keyboard/Mouse interactions: check "what's this?" on the scene 3D to get all interaction shortcuts.
   *
-  * There are five default InteractiveViewer "singleton" instances named:
-  * - "3DViewer" classical 3D geometry viewer (GEOMETRY_VIEWER)
-  * - "axialViewer" to display axial slice (SLICE_VIEWER)
-  * - "coronalViewer" to display axial slice (SLICE_VIEWER)
-  * - "sagittalViewer" to display sagittal slice (SLICE_VIEWER)
-  * - "arbitraryViewer" to display an arbitrary slice (SLICE_VIEWER)
-  * They can be accessed using the corresponding get methods.
-  * This is why there are no public constructor for this class.
-  * To create a new custom instance of this class, use the getNewViewer(..) method.
-  * Of course you can create as many InteractiveViewer instance as you like, but if you try to create
-  * a InteractiveViewer with a name that match one of the default InteractiveViewer name, getNewViewer(..) automatically
-  * will return the corresponding default InteractiveViewer instance.
-  * The list of InteractiveViewer instances is kept in this class as well.
-  *
-  * InteractiveViewer class is completely independent from MainWindow application skeleton (and please keep it that way!)
-  *
   * InteractiveViewer manages a list of cameras. Each camera has a name. Use getCamera(QString) to
   * create a new camera, or get the default camera and setActiveCamera(QString) to change the active
   * camera.
@@ -139,24 +111,24 @@ protected:
 
 class CAMITK_API InteractiveViewer : public Viewer {
     Q_OBJECT
-    Q_ENUMS(HighlightMode RendererWidget::ControlMode RendererWidget::CameraOrientation)    // so that it can be used in property editor
+    Q_ENUMS(HighlightMode RendererWidget::ControlMode RendererWidget::CameraOrientation);    // so that it can be used in property editor
 
 public:
     /// \enum ViewerType there is two possibilities: this InteractiveViewer is used to display slices or geometry
     enum ViewerType {
         SLICE_VIEWER,     ///< display slices (the view is blocked in 2D and the slider is available)
-        GEOMETRY_VIEWER  ///< display 3D stuff (geometry, etc...)
+        GEOMETRY_VIEWER   ///< display 3D stuff (geometry, etc...)
     };
 
     /** \enum PickingMode Different kind of picking must be available: pixel in slice, a point, a cell, ...
      *    So far, only pixel picking is implemented. */
     enum PickingMode {
-        PIXEL_PICKING,  ///< pick a pixel on a Slice
-        POINT_PICKING,  ///< pick a point in the VTK representation of an Geometry
-        CELL_PICKING,   ///< pick a cell in the VTK representation of an Geometry
-        AREA_CELL_PICKING, ///< pick cells that are inside a rectangular area in the VTK representation of an Geometry
+        PIXEL_PICKING,      ///< pick a pixel on a Slice
+        POINT_PICKING,      ///< pick a point in the VTK representation of an Geometry
+        CELL_PICKING,       ///< pick a cell in the VTK representation of an Geometry
+        AREA_CELL_PICKING,  ///< pick cells that are inside a rectangular area in the VTK representation of an Geometry
         AREA_POINT_PICKING, ///< pick points that are inside a rectangular area in the VTK representation of an Geometry
-        NO_PICKING     ///< no picking possible
+        NO_PICKING          ///< no picking possible
     };
 
     /** \enum HighlightMode describes the current mode of display. It is useful to change the
@@ -166,40 +138,18 @@ public:
     enum HighlightMode {
         OFF,            ///< both selected and non-selected Components are in default mode
         SELECTION,      ///< the selected Components are in default mode, the non-selected Components are shaded
-        SELECTION_ONLY ///< the selected Components are in default mode, the non-selected are hidden
+        SELECTION_ONLY  ///< the selected Components are in default mode, the non-selected are hidden
     };
 
-    /** @name General/Singleton
-     *  There is no public constructor for this class, but 5 default instances.
-     */
     /// @{
-    /** Use this method to create a new custom instance of this class.
-      * Of course you can create as many InteractiveViewer instance as you like, but if you try to create
-      * a InteractiveViewer with a name that match one of the default InteractiveViewer name, getNewViewer(..) automatically
-      * will return the corresponding default InteractiveViewer instance.
-      */
-    static InteractiveViewer* getNewViewer(QString, ViewerType type);
-
-    /// get a viewer by its name (return NULL if no instance has the given name, use getNewViewer to create one viewer)
-    static InteractiveViewer* getViewer(QString);
-
-    /// get the 3D viewer default InteractiveViewer (provided for convenience, equivalent to getViewer("3DViewer")
-    static InteractiveViewer* get3DViewer();
-
-    /// get the axial default InteractiveViewer (provided for convenience, equivalent to getViewer("axialViewer")
-    static InteractiveViewer* getAxialViewer();
-
-    /// get the coronal default InteractiveViewer (provided for convenience, equivalent to getViewer("coronalViewer")
-    static InteractiveViewer* getCoronalViewer();
-
-    /// get the sagittal default InteractiveViewer (provided for convenience, equivalent to getViewer("sagittalViewer")
-    static InteractiveViewer* getSagittalViewer();
-
-    /// get the arbitrary default InteractiveViewer (provided for convenience, equivalent to getViewer("arbitraryViewer")
-    static InteractiveViewer* getArbitraryViewer();
+    /** Construtor.
+     *  @param name    the name of the viewer is mandatory, it is used as an identifier (e.g. in MedicalImageViewer)
+     *  @param type    type of the InteractiveViewer, depending on which it will behave as slice viewer, i.e. with no rotation interactions are possible, or 3D viewer
+     */
+    Q_INVOKABLE InteractiveViewer(QString& name, camitk::InteractiveViewer::ViewerType type);
 
     /** Destructor */
-    ~InteractiveViewer() override;
+    virtual ~InteractiveViewer() override;
 
     /// get the scene name
     QString getName() const;
@@ -208,14 +158,11 @@ public:
     /** @name Inherited from Viewer
       */
     ///@{
-    /// returns the number of Component that are displayed by this viewer
-    unsigned int numberOfViewedComponent() override;
-
     /// Refresh the display.
     void refresh(Viewer* whoIsAsking = nullptr) override;
 
     /// get the InteractiveViewer widget (QTreeWidget). @param parent the parent widget for the viewer widget
-    QWidget* getWidget(QWidget* parent) override;
+    QWidget* getWidget() override;
 
     /// get the  InteractiveViewer propertyObject (only non-null for GEOMETRY_VIEWER)
     QObject* getPropertyObject() override;
@@ -260,21 +207,16 @@ public:
     /// set background color
     virtual void setBackgroundColor(QColor);
 
+public slots:
     /** @name Viewing/Interaction Property
      */
     ///@{
-public slots:
     /// set the backface culling mode (default is true).
     void setBackfaceCulling(bool);
 
     /// visibility of the screenshot in slice viewers
     void setScreenshotAction(bool);
 
-public:
-    /// get the backface current culling mode.
-    // bool getBackfaceCulling() const;
-
-public slots:
     /** Update the visualization of lines (for all the InterfaceGeometry of the scene).
       *
       * \note it is only possible to transform lines to tubes if
@@ -288,19 +230,17 @@ public:
     /// Handle keyboard events in the scene, let to the parent widget if not processed here. This method is a friend of class InteractiveViewerFrame
     void keyPressEvent(QKeyEvent* e);
 
-    /// Set the current highlighting mode.
-    virtual void setHighlightMode(InteractiveViewer::HighlightMode);
+    /// Set the current highlighting mode from the current value of the property
+    virtual void setHighlightMode();
 
     /// return interactiveViewer RendererWidget
     inline RendererWidget* getRendererWidget() {
         return rendererWidget;
     }
 
-
     ///@}
 
-    /** @name Misc
-    */
+    /// @name Misc
     /// @{
     /// set the color scale in the viewport, use setColorScaleMinMax / setColorScaleTitle to change the displayed values.
     void setColorScale(bool);
@@ -335,6 +275,8 @@ public:
 
 public slots:
 
+    /// @name public slots
+    /// @{
     /** Slot called when the InteractiveViewer slider has been changed. If there is a InterfaceBitMap in the scene,
      *    set the slice index with the new slider value. */
     void sliderChanged(int);
@@ -348,27 +290,14 @@ public slots:
     /** Slot called when the InteractiveViewer z angle update has been changed. */
     void zAngleChanged(double angle);
 
-    /// show/hide the copyright in 3D
-    void toggleCopyright(bool);
-
-protected:
-    /** @name General/Singleton
-     */
-    /// @{
-    /** Construtor.
-     *  @param name    the name of the scene is mandatory, it is used as an identifier (e.g. in MedicalImageViewer)
-     *  @param type    type of the InteractiveViewer, depending on which it will behave as slice viewer, i.e. with no rotation interactions are possible, or 3D viewer
-     */
-    InteractiveViewer(QString& name, ViewerType type);
-
-    /// the map containing all the InteractiveViewer instances
-    static QMap<QString, InteractiveViewer*> viewers;
-
-    /// the QString array containing all InteractiveViewer instance default names
-    static QString defaultNames[5];
-
+    /// show/hide the logo at the bottom right corner
+    void toggleLogo(bool);
     /// @}
 
+protected:
+
+    /// used by both constructors
+    void init();
 
     /** @name Display properties
       *
@@ -391,6 +320,9 @@ protected:
     /// for InterfaceBitMap, toggle the interpolation mode (intern method, not a property because it can only be modified by the keyboard interaction)
     void toggleInterpolation();
 
+    /// for InterfaceBitMap, reset the lut that was changed by the image interactor (window and level)
+    void resetLUT();
+
     /// Update the display of the given Component, according to its selection state and the current HighlightMode.
     void updateSelectionDisplay(Component*);
 
@@ -410,8 +342,7 @@ protected:
     QMap<QString, vtkSmartPointerCamera> cameraMap;
     ///@}
 
-    /** @name Widget/Action management
-      */
+    /// @name Widget/Action management
     /// @{
     /// The 3D scene itself, wrapping VTK render window, renderer and interactor in a single Qt widget
     RendererWidget* rendererWidget;
@@ -436,7 +367,7 @@ protected:
     QMenu* viewerMenu;
 
     /// the QToolBar for the InteractiveViewer
-    QToolBar* viewerToolBar;
+    QToolBar* viewerToolbar;
 
     /// the ComboBox for mesh scalar data
     QComboBox* scalarDataComboBox;
@@ -485,7 +416,7 @@ protected:
     QAction* toggleOrientationDecorationsAction;
 
     /// button to remove the copyright
-    QAction* toggleCopyrightAction;
+    QAction* toggleLogoAction;
 
     /// button allows one to display the labels of the object3D
     QAction* toggleLabelAction;
