@@ -51,6 +51,7 @@
 #include <QHBoxLayout>
 #include <QMenu>
 #include <QKeyEvent>
+#include <QRegularExpressionValidator>
 #include <QApplication>
 #include <QLabel>
 #include <QToolButton>
@@ -60,6 +61,7 @@
 #include <QStyleOption>
 #include <QPainter>
 #include <QMap>
+#include <QRegularExpression>
 
 #if defined(Q_CC_MSVC)
 #    pragma warning(disable: 4786) /* MS VS 6: truncating debug info after 255 characters */
@@ -879,7 +881,7 @@ class QtLineEditFactoryPrivate : public EditorFactoryPrivate<QLineEdit> {
 public:
 
     void slotPropertyChanged(QtProperty* property, const QString& value);
-    void slotRegExpChanged(QtProperty* property, const QRegExp& regExp);
+    void slotRegExpChanged(QtProperty* property, const QRegularExpression& regExp);
     void slotSetValue(const QString& value);
     void slotEchoModeChanged(QtProperty*, int);
 };
@@ -900,7 +902,7 @@ void QtLineEditFactoryPrivate::slotPropertyChanged(QtProperty* property,
 }
 
 void QtLineEditFactoryPrivate::slotRegExpChanged(QtProperty* property,
-        const QRegExp& regExp) {
+        const QRegularExpression& regExp) {
     if (!m_createdEditors.contains(property)) {
         return;
     }
@@ -917,7 +919,7 @@ void QtLineEditFactoryPrivate::slotRegExpChanged(QtProperty* property,
         const QValidator* oldValidator = editor->validator();
         QValidator* newValidator = nullptr;
         if (regExp.isValid()) {
-            newValidator = new QRegExpValidator(regExp, editor);
+            newValidator = new QRegularExpressionValidator(regExp, editor);
         }
         editor->setValidator(newValidator);
         if (oldValidator) {
@@ -998,8 +1000,8 @@ QtLineEditFactory::~QtLineEditFactory() {
 void QtLineEditFactory::connectPropertyManager(QtStringPropertyManager* manager) {
     connect(manager, SIGNAL(valueChanged(QtProperty*, const QString&)),
             this, SLOT(slotPropertyChanged(QtProperty*, const QString&)));
-    connect(manager, SIGNAL(regExpChanged(QtProperty*, const QRegExp&)),
-            this, SLOT(slotRegExpChanged(QtProperty*, const QRegExp&)));
+    connect(manager, SIGNAL(regExpChanged(QtProperty*, QRegularExpression)),
+            this, SLOT(slotRegExpChanged(QtProperty*, QRegularExpression)));
     connect(manager, SIGNAL(echoModeChanged(QtProperty*, int)),
             this, SLOT(slotEchoModeChanged(QtProperty*, int)));
 }
@@ -1014,9 +1016,9 @@ QWidget* QtLineEditFactory::createEditor(QtStringPropertyManager* manager,
 
     QLineEdit* editor = d_ptr->createEditor(property, parent);
     editor->setEchoMode((EchoMode)manager->echoMode(property));
-    QRegExp regExp = manager->regExp(property);
-    if (regExp.isValid()) {
-        QValidator* validator = new QRegExpValidator(regExp, editor);
+    QRegularExpression regExp = manager->regExp(property);
+    if (regExp.isValid() && !regExp.pattern().isEmpty()) {
+        QValidator* validator = new QRegularExpressionValidator(regExp, editor);
         editor->setValidator(validator);
     }
     editor->setText(manager->value(property));
@@ -1036,8 +1038,8 @@ QWidget* QtLineEditFactory::createEditor(QtStringPropertyManager* manager,
 void QtLineEditFactory::disconnectPropertyManager(QtStringPropertyManager* manager) {
     disconnect(manager, SIGNAL(valueChanged(QtProperty*, const QString&)),
                this, SLOT(slotPropertyChanged(QtProperty*, const QString&)));
-    disconnect(manager, SIGNAL(regExpChanged(QtProperty*, const QRegExp&)),
-               this, SLOT(slotRegExpChanged(QtProperty*, const QRegExp&)));
+    disconnect(manager, SIGNAL(regExpChanged(QtProperty*, QRegularExpression)),
+               this, SLOT(slotRegExpChanged(QtProperty*, QRegularExpression)));
     disconnect(manager, SIGNAL(echoModeChanged(QtProperty*, int)),
                this, SLOT(slotEchoModeChanged(QtProperty*, int)));
 }
@@ -1520,7 +1522,7 @@ QtCharEdit::QtCharEdit(QWidget* parent)
     : QWidget(parent),  m_lineEdit(new QLineEdit(this)) {
     auto* layout = new QHBoxLayout(this);
     layout->addWidget(m_lineEdit);
-    layout->setMargin(0);
+    layout->setContentsMargins(QMargins());
     m_lineEdit->installEventFilter(this);
     m_lineEdit->setReadOnly(true);
     m_lineEdit->setFocusProxy(this);
@@ -1532,7 +1534,7 @@ bool QtCharEdit::eventFilter(QObject* o, QEvent* e) {
     if (o == m_lineEdit && e->type() == QEvent::ContextMenu) {
         auto* c = static_cast<QContextMenuEvent*>(e);
         QMenu* menu = m_lineEdit->createStandardContextMenu();
-        QList<QAction*> actions = menu->actions();
+        const auto actions = menu->actions();
         QListIterator<QAction*> itAction(actions);
         while (itAction.hasNext()) {
             QAction* action = itAction.next();
@@ -1961,8 +1963,7 @@ public:
     bool m_updatingEnum{false};
 };
 
-QtCursorEditorFactoryPrivate::QtCursorEditorFactoryPrivate()
-{
+QtCursorEditorFactoryPrivate::QtCursorEditorFactoryPrivate() {
 
 }
 
@@ -2161,11 +2162,10 @@ void QtColorEditWidget::setValue(const QColor& c) {
 }
 
 void QtColorEditWidget::buttonClicked() {
-    bool ok = false;
     QRgb oldRgba = m_color.rgba();
-    QRgb newRgba = QColorDialog::getRgba(oldRgba, &ok, this);
-    if (ok && newRgba != oldRgba) {
-        setValue(QColor::fromRgba(newRgba));
+    QColor newRgba = QColorDialog::getColor(oldRgba, this, "Choose New Color",  QColorDialog::ShowAlphaChannel);
+    if (newRgba.isValid() && newRgba.rgba() != oldRgba) {
+        setValue(QColor::fromRgba(newRgba.rgba()));
         emit valueChanged(m_color);
     }
 }
